#ifndef __Function_h__
#define __Function_h__

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS



#define HST_TYPE_FUNCTION                  (hst_function_get_type ())
#define HST_FUNCTION(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), HST_TYPE_FUNCTION, HstFunction))
#define HST_FUNCTION_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass),  HST_TYPE_FUNCTION, HstFunctionClass))
#define HST_IS_FUNCTION(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HST_TYPE_FUNCTION))
#define HST_IS_FUNCTION_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass),  HST_TYPE_FUNCTION))
#define HST_FUNCTION_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj),  HST_TYPE_FUNCTION, HstFunctionClass))

typedef struct _HstTuple HstTuple;
typedef struct _HstPoint HstPoint;
struct _HstTuple {
    int n;
    HstPoint *P;
};
struct _HstPoint {
    double x;
    double y;
};


typedef struct _HstFunction HstFunction;
typedef struct _HstFunctionClass HstFunctionClass;

struct _HstFunction {
    GObject parent_instance;
    double (*func)(double val);
    int    nbin;
    double xmin,xmax,step;
    gchar *name;
};

/**
 * HstFunctionClass:
 * @eval: this is the method that evaluates the function for a given value
 * @array: this methid provides a tuple for drawing the function
 * @resize: this methid is called when the display widget needs to change size
 * @print:
 */
struct _HstFunctionClass {
    GObjectClass parent_class;
    
    double    (*eval)(HstFunction *f, double val);
    void      (*array)(HstFunction *f, HstTuple *tuple, double xmin, double xmax);
    int       (*resize)(HstFunction *f,int width, int height);
    GString  *(*print)(HstFunction *f);
    /* signals */
    
    void (*range_changed)(HstFunction *f);
};
typedef double (*HstGFunction1D)(double val);

GType      hst_function_get_type (void) G_GNUC_CONST;
HstFunction *hst_function_new(int nx, double x_min, double x_max, HstGFunction1D  ptr);
double     hst_function_eval(HstFunction *f,double val);
int        hst_function_get_npts(HstFunction *f);
double     hst_function_get_xmin(HstFunction *f);
double     hst_function_get_xmax(HstFunction *f);
double     hst_function_get_step(HstFunction *f);
int        hst_function_find_bin(HstFunction *f, double x);
double     hst_function_bin_center(HstFunction *f, int ib);
int        hst_function_resize(HstFunction *f,int width,int height);
gchar     *hst_function_get_name(HstFunction *f);
void       hst_function_set_name(HstFunction *f, gchar *name);
void       hst_function_get_array(HstFunction *f, HstTuple *t, double xmin, double xmax);
GString   *hst_function_print(HstFunction *f);
GString   *hst_function_print_list(GList *list_of_functions);
void       hst_function_clear_tuple(HstTuple *t);
G_END_DECLS
#endif
