#ifndef __HstHistogram_h__
#define __HstHistogram_h__

#include <gtkhst/function.h>
#include <gtkhst/stats.h>

G_BEGIN_DECLS

#define HST_TYPE_HISTOGRAM                  (hst_histogram_get_type ())
#define HST_HISTOGRAM(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), HST_TYPE_HISTOGRAM, HstHistogram))
#define HST_HISTOGRAM_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass),  HST_TYPE_HISTOGRAM, HstHistogramClass))
#define HST_IS_HISTOGRAM(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HST_TYPE_HISTOGRAM))
#define HST_IS_HISTOGRAM_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass),  HST_TYPE_HISTOGRAM))
#define HST_HISTOGRAM_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj),  HST_TYPE_HISTOGRAM, HstHistogramClass))

typedef struct _HstHistogram HstHistogram;
typedef struct _HstHistogramClass HstHistogramClass;
struct _HstHistogram {
    HstFunction parent_instance;
    
    double *y;       /* array for bin contents */
    int     owner;   /* flag saying if we own the array */
    HstStats *stat;     /* statistics */

    double vmin,vmax; /* max. and min. values of bin contents */
};

struct _HstHistogramClass {
    HstFunctionClass parent_class;

    /* Functions */
    void (*fill)(HstHistogram *hst, double val, double weight);
    void (*reset)(HstHistogram *hst);

    /* signals */
    void (*changed)(HstHistogram *hst);
};

GType         hst_histogram_get_type (void) G_GNUC_CONST;
HstHistogram *hst_histogram_new(gint n, gdouble xmin, gdouble xmax, gdouble *ptr);
void          hst_histogram_fill(HstHistogram *hst,double val, double weight);
void          hst_histogram_reset(HstHistogram *hst);
void          hst_histogram_update_stats(HstHistogram *hst);
gdouble       hst_histogram_mean(HstHistogram *hst);
gdouble       hst_histogram_rms(HstHistogram *hst);
gdouble       hst_histogram_stdev(HstHistogram *hst);
gdouble       hst_histogram_entries(HstHistogram *hst);
gdouble       hst_histogram_min_value(HstHistogram *hst);
gdouble       hst_histogram_max_value(HstHistogram *hst);
gdouble       hst_histogram_vmin(HstHistogram *hst);
gdouble       hst_histogram_vmax(HstHistogram *hst);
gdouble      *hst_histogram_get_data(HstHistogram *hst);
void          hst_histogram_resize(HstHistogram *hst, int nbin, double xmin, double xmax);
gint          hst_histogram_get_stats(HstHistogram *hst, HstStats *s);
void          hst_histogram_get_buffer(HstHistogram *hst, int *sz, double **buffer);
G_END_DECLS
#endif
