#ifndef __hst_chart1d_h__
#define __hst_chart1d_h__

#include <gtk/gtk.h>
#include <gtkhst/function.h>

G_BEGIN_DECLS
                                                                                
#define HST_TYPE_CHART1D                  (hst_chart1d_get_type ())
#define HST_CHART1D(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), HST_TYPE_CHART1D, HstChart1D))
#define HST_CHART1D_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), HST_TYPE_CHART1D, HstChart1DClass))
#define HST_IS_CHART1D(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HST_TYPE_CHART1D))
#define HST_IS_CHART1D_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), HST_TYPE_CHART1D))
#define HST_CHART1D_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), HST_TYPE_CHART1D, HstChart1DClass))

typedef enum {
    HST_COLOR_FOREGROUND = 0,
    HST_COLOR_BACKGROUND,
    HST_COLOR_AXIS,
    HST_COLOR_CURVE,
    HST_NCOLORS
}  HstChartColors;
GType hst_chart_colors_get_type (void);
#define HST_TYPE_CHART_COLORS (hst_chart_colors_get_type())

typedef enum {
   HST_AXIS_NONE   = 0,
   HST_AXIS_LEFT   = 1,
   HST_AXIS_MIDDLE = 2,
   HST_AXIS_RIGHT  = 4,
   HST_AXIS_X_MASK = 0x0f,
   HST_AXIS_Y_MASK = 0xf0,
   HST_AXIS_Z_MASK = 0xf00
} HstChartAxisPos;
GType hst_chart_axis_pos_get_type (void);
#define HST_TYPE_CHART_AXIS_POS (hst_chart_axis_pos_get_type())


typedef enum {
   HST_FILLED = 1,
   HST_LOGX   = 2,
   HST_LOGY   = 4,
   HST_STATS  = 8
} HstChartAttr;
GType hst_chart_attr_get_type(void);
#define HST_TYPE_CHART_ATTR ( hst_chart_attr_get_type() )

typedef enum {
    HST_X_AXIS,
    HST_Y_AXIS,
    HST_Z_AXIS
} HstChartAxisType;
GType hst_chart_axis_type(void);
#define HST_TYPE_CHART_AXIS_TYPE ( hst_chart_axis_type() )


typedef struct _HstChart1D       HstChart1D;
typedef struct _HstChart1DClass  HstChart1DClass;
typedef struct _HstCanvas        HstCanvas;
typedef struct _HstChartEvent    HstChartEvent;
struct _HstChart1D {
    GtkDrawingArea obj;
    HstCanvas     *cnvs;
    HstFunction   *func;
    GdkColor       colors[HST_NCOLORS]; /* =0 foreground =1 background =2 axis */
    GList         *curves;
    HstChartEvent *evt;
    HstTuple         *xpt;
    char          *xtitle,*ytitle;
    
    
    int           tmout;
    int           sync_id;
    unsigned long drawaxis;
    double        xmin,xmax;
    double        ymin,ymax;
    double        vmin,vmax;
    gboolean      colors_changed;
    guint         attr;
};

/**
 * HstChart1DClass:
 * @draw_pixmap: draws the curves associated witht the widget
 * @draw_frame:  draws the whole frame
 * @draw_stats:  draws the stats
 * @draw_axis: draws the function axis
 * @mouse:
 * @xtra_menu: provides additional items for the pop-up menu
 *
 * An object deriving from HstChart1D has defined a number
 * of methods which are called in the process of drawing
 * the object.
 *
 */
struct _HstChart1DClass {
    GtkDrawingAreaClass parent_class;
      
    void (*draw_pixmap) (HstChart1D *h);
    void (*draw_frame) (HstChart1D *h);
    void (*draw_stats) (HstChart1D *h);
    void (*draw_axis) (HstChart1D *h);
    void (* mouse ) (HstChart1D *h);
    void (*xtra_menu)(HstChart1D *h, GtkWidget *w);
};

GType      hst_chart1d_get_type (void) G_GNUC_CONST;
GtkWidget *hst_chart1d_new(HstFunction *ff);
void       hst_chart1d_add_curve(HstChart1D *h, HstFunction *f);
void       hst_chart1d_remove_curves(HstChart1D *h);
void       hst_chart1d_monitor_start (HstChart1D *h, int tim);
void       hst_chart1d_monitor_stop (HstChart1D *h);
void       hst_chart1d_draw_x_axis (HstChart1D *h, HstChartAxisPos val);
void       hst_chart1d_draw_y_axis (HstChart1D *h, HstChartAxisPos val);
gchar      *hst_chart1d_get_xtitle (HstChart1D *hst);
void       hst_chart1d_set_xtitle (HstChart1D *hst, const char *tit);
gchar      *hst_chart1d_get_ytitle (HstChart1D *hst);
void       hst_chart1d_set_ytitle (HstChart1D *hst, const char *tit);
void       hst_chart1d_filled (HstChart1D *h, gboolean z);
gboolean   hst_chart1d_is_filled (HstChart1D *h);
void       hst_chart1d_logx (HstChart1D *h, gboolean z);
gboolean   hst_chart1d_is_logx (HstChart1D *h);
void       hst_chart1d_logy (HstChart1D *h, gboolean z);
gboolean   hst_chart1d_is_logy (HstChart1D *h);
void       hst_chart1d_show_stats(HstChart1D *h, gboolean z);
gboolean   hst_chart1d_stats(HstChart1D *h);

void         hst_chart1d_set_curve_colors(HstChart1D *h,...);
void         hst_chart1d_add_curve_color(HstChart1D *h, gchar *color_name);
int          hst_chart1d_get_n_func(HstChart1D *h);
HstFunction *hst_chart1d_get_func(HstChart1D *h, int n);
void         hst_chart1d_set_func(HstChart1D *h, HstFunction *f);
void         hst_chart1d_sync_range(HstChart1D *h, gboolean z);
int          hst_chart1d_print(HstChart1D *h, const char *file_name, const char *type);
/*
 * Sets the colors for:
 *   - foreground
 *   - background
 *   - axis
 * The values should be any accepted by gdk_color_parser
 * The last argument should be NULL
 *
 *   hst_chart1d_set_colors(h,"foreground","black","axis","red",0)
 */
void       hst_chart1d_set_colors(HstChart1D *h,...);
G_END_DECLS
#endif
