#ifndef PYTHONLOADER_H_
#define PYTHONLOADER_H_

#ifdef HAVE_CONFIG_H
#ifdef _WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#if defined(HAS_PYTHON)
#include <Python.h>
#include "python-compat.h"
#include <map>
#include <string>


/**
 * This class handles the interaction with python
 */
class PythonLoader
{
    private:
        bool _initialized;
    protected:
        PythonLoader();
        PythonLoader(const PythonLoader &);

        typedef std::map<std::string, PyObject *> ObjectList;
        ObjectList ModuleList;
        ObjectList MethodList;
        static PythonLoader __the_loader;
        PyThreadState *_save;

    public:
        static PythonLoader &getInstance();
        virtual ~PythonLoader();

        void allow_threads();
        void disable_threads();
        void wait(double seconds);
        PyObject *load_module(const std::string &module, const std::string &method);
        PyObject *execute_function_from_module(const char *module_name, const char *function_name, bool export_funcs, PyObject **mod=0, PyObject **meth=0);
        void initialize(int argc, char **argv);
        void add_path(const char *p);
        void show_path();
        int export_functions(PyObject *);
};
void print_python_error();
void add_python_path(const char *p);
#endif

#endif /*PYTHONLOADER_H_*/
