/*
 * PythonObjects.cc
 *
 *  Created on: Jun 1, 2010
 *      Author: lacasta
 */
#ifdef HAVE_CONFIG_H
#ifdef _WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#if defined(HAS_PYTHON)

/*
 * The DAQconfig
 */
#include <Python.h>
#include <iostream>
#include <structmember.h>

#include <ConfigFactory.h>
#include <DAQConfig.h>
#include <Analysis.h>
int set_parameter_value(const std::string &name, const std::string &value);

typedef struct {
    PyObject_HEAD
    /* Type-specific fields go here. */
    DAQConfig *cfg;
} alibava_DAQconfig;

static void
daqconfig_dealloc(alibava_DAQconfig* self)
{
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject *
daqconfig_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    alibava_DAQconfig *self;

    self = (alibava_DAQconfig *)type->tp_alloc(type, 0);
    if (self)
    {
        self->cfg = 0;
    }
    return (PyObject *)self;
}

static int
daqconfig_init(alibava_DAQconfig *self, PyObject *args, PyObject *kwds)
{
    if (!self->cfg)
        self->cfg = dynamic_cast<DAQConfig *>(ConfigFactory::instance()->get("daq"));

    return 0;
}

static PyObject *
daqconfig_get_chan(alibava_DAQconfig *self, void *closure)
{
    PyObject *rc = PyInt_FromLong(self->cfg->chan());
    return rc;
}
static PyObject *
daqconfig_get_nevt(alibava_DAQconfig *self, void *closure)
{
    PyObject *rc = PyInt_FromLong(self->cfg->sample_size());
    return rc;
}
static PyObject *
daqconfig_get_nevts(alibava_DAQconfig *self, void *closure)
{
    PyObject *rc = PyInt_FromLong(self->cfg->nevts());
    return rc;
}
static PyObject *
daqconfig_get_delay(alibava_DAQconfig *self, void *closure)
{
    PyObject *rc = PyInt_FromLong(self->cfg->delay());
    return rc;
}
static int
daqconfig_set_chan(alibava_DAQconfig *self, PyObject *value, void *closure)
{
    if (! PyInt_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "DAQconfig: chan must be an intger");
        return -1;
    }
    self->cfg->chan(PyInt_AsLong(value));
    return 0;
}
static int
daqconfig_set_nevt(alibava_DAQconfig *self, PyObject *value, void *closure)
{
    if (! PyInt_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "DAQconfig: sample_size must be an intger");
        return -1;
    }
    self->cfg->sample_size(PyInt_AsLong(value));
    return 0;
}
static int
daqconfig_set_nevts(alibava_DAQconfig *self, PyObject *value, void *closure)
{
    if (! PyInt_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "DAQconfig: nevts must be an intger");
        return -1;
    }
    self->cfg->nevts(PyInt_AsLong(value));
    return 0;
}
static int
daqconfig_set_delay(alibava_DAQconfig *self, PyObject *value, void *closure)
{
    if (! PyInt_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "DAQconfig: delay must be an intger");
        return -1;
    }
    self->cfg->delay(PyInt_AsLong(value));
    return 0;
}



static PyGetSetDef daqconfig_getseters[] =
{
 {(char *)"chan", (getter)daqconfig_get_chan, (setter)daqconfig_set_chan,(char *)"", NULL},
 {(char *)"sample_size", (getter)daqconfig_get_nevt, (setter)daqconfig_set_nevt,(char *)"", NULL},
 {(char *)"nevts", (getter)daqconfig_get_nevts, (setter)daqconfig_set_nevts,(char *)"", NULL},
 {(char *)"delay", (getter)daqconfig_get_delay, (setter)daqconfig_set_delay,(char *)"", NULL},
 {NULL}  /* Sentinel */
};

static PyTypeObject DAQconfig_Type = {
    PyObject_HEAD_INIT(NULL)
    0,                                /*ob_size*/
    "DAQconfig",                      /*tp_name*/
    sizeof(alibava_DAQconfig),        /*tp_basicsize*/
    0,                                /*tp_itemsize*/
    (destructor)daqconfig_dealloc,    /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,                         /*tp_compare*/
    0,                         /*tp_repr*/
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    0,                         /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT,        /*tp_flags*/
    "DAQconfig object",        /* tp_doc */
    0,                     /* tp_traverse */
    0,                     /* tp_clear */
    0,                     /* tp_richcompare */
    0,                     /* tp_weaklistoffset */
    0,                     /* tp_iter */
    0,                     /* tp_iternext */
    0,             /* tp_methods */
    0,             /* tp_members */
    daqconfig_getseters,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)daqconfig_init,      /* tp_init */
    0,                         /* tp_alloc */
    daqconfig_new,                 /* tp_new */

};





/*********************************************************************************
 * Analysis
 *********************************************************************************/

typedef struct {
    PyObject_HEAD
    /* Type-specific fields go here. */
    Analysis *ana;
} alibava_Analysis;

static void
analysis_dealloc(alibava_Analysis* self)
{
    self->ob_type->tp_free((PyObject*)self);
}

static PyObject *
analysis_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    alibava_Analysis *self;

    self = (alibava_Analysis *)type->tp_alloc(type, 0);
    if (self)
    {
        self->ana = 0;
    }
    return (PyObject *)self;
}

static int
analysis_init(alibava_Analysis *self, PyObject *args, PyObject *kwds)
{
    if (!self->ana)
        self->ana = dynamic_cast<Analysis *>(ConfigFactory::instance()->get("analysis"));

    return 0;
}

static PyObject *get_tuple(int n, double *v)
{
    int i;
    PyObject *rc = PyTuple_New(n);
    for (i=0; i<n; i++)
    {
        PyTuple_SET_ITEM(rc, i, PyFloat_FromDouble(v[i]));
    }
    return rc;
}

static PyObject *
analysis_get_ped(alibava_Analysis *self, void *closure)
{
    return get_tuple(256, self->ana->pedestals());
}
static PyObject *
analysis_get_noise(alibava_Analysis *self, void *closure)
{
    return get_tuple(256, self->ana->noise());
}
static PyObject *
analysis_get_signal(alibava_Analysis *self, void *closure)
{
    return get_tuple(256, self->ana->signal());
}
static PyObject *
analysis_get_raw(alibava_Analysis *self, void *closure)
{
    return get_tuple(256, self->ana->raw());
}
static PyObject *
analysis_get_sn(alibava_Analysis *self, void *closure)
{
    return get_tuple(256, self->ana->sn());
}
static PyObject *
analysis_get_nhits(alibava_Analysis *self, void *closure)
{
    return PyInt_FromLong( self->ana->nhits() );
}

static PyObject *
analysis_get_hits(alibava_Analysis *self, void *closure)
{
    int ihit, nhit=self->ana->nhits();
    PyObject *rc = PyTuple_New(nhit);
    for (ihit=0; ihit<nhit; ihit++)
    {
        PyObject *pyhit = PyTuple_New(2);
        PyTuple_SET_ITEM(rc, ihit, pyhit);
        PyTuple_SET_ITEM(pyhit, 0, PyInt_FromLong(self->ana->hit(ihit).first));
        PyTuple_SET_ITEM(pyhit, 1, PyInt_FromLong(self->ana->hit(ihit).second));
    }
    return rc;
}

static PyGetSetDef analysis_getseters[] =
{
 {(char *)"hits", (getter)analysis_get_hits, NULL,(char *)"", NULL},
 {(char *)"nhits", (getter)analysis_get_nhits, NULL,(char *)"", NULL},
 {(char *)"noise", (getter)analysis_get_noise, NULL,(char *)"", NULL},
 {(char *)"ped", (getter)analysis_get_ped, NULL,(char *)"", NULL},
 {(char *)"raw", (getter)analysis_get_raw, NULL,(char *)"", NULL},
 {(char *)"signal", (getter)analysis_get_signal, NULL,(char *)"", NULL},
 {(char *)"sn", (getter)analysis_get_sn, NULL,(char *)"", NULL},
 {NULL}  /* Sentinel */
};

static PyTypeObject analysis_Type = {
    PyObject_HEAD_INIT(NULL)
    0,                                /*ob_size*/
    "Analysis",                      /*tp_name*/
    sizeof(alibava_Analysis),        /*tp_basicsize*/
    0,                                /*tp_itemsize*/
    (destructor)analysis_dealloc,    /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,                         /*tp_compare*/
    0,                         /*tp_repr*/
    0,                         /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    0,                         /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT,        /*tp_flags*/
    "Analysis object",        /* tp_doc */
    0,                     /* tp_traverse */
    0,                     /* tp_clear */
    0,                     /* tp_richcompare */
    0,                     /* tp_weaklistoffset */
    0,                     /* tp_iter */
    0,                     /* tp_iternext */
    0,             /* tp_methods */
    0,             /* tp_members */
    analysis_getseters,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)analysis_init,      /* tp_init */
    0,                         /* tp_alloc */
    analysis_new,                 /* tp_new */

};


/*
 * Parameter setting
 */

PyDoc_STRVAR(paramconfig_setpar_doc,
"ParamConfig.set_par(name,value)\n\
Sets the value of a 'named' parameter of the Beetle.\n\
");
static PyObject *
paramconfig_set_par(PyObject *self, PyObject *args, PyObject *kwargs)
{
    char *name, *val;
    PyObject *value, *ostr;
    if (!PyArg_ParseTuple(args,"sO:ParamConfig.set_par",&name,&value))
          return NULL;


    ostr = PyObject_Str(value);
    val = PyString_AsString( ostr );

    set_parameter_value(name, val);

    Py_DecRef(ostr);
    Py_INCREF(Py_None);
    return Py_None;
}


static PyMethodDef module_methods[] = {
   {"set_par",(PyCFunction)paramconfig_set_par,METH_VARARGS,paramconfig_setpar_doc},
   {NULL}  /* Sentinel */
};

void init_python_objects()
{
    std::cout << "Initializing Alibava Python objects" << std::endl;
    PyObject *m = Py_InitModule3("alibava", module_methods,"The alibava objects");

    if (PyType_Ready(&DAQconfig_Type) < 0)
        return;

    if (PyType_Ready(&analysis_Type) < 0)
        return;

    Py_INCREF(&DAQconfig_Type);
    PyModule_AddObject(m, "DAQConfig", (PyObject *)&DAQconfig_Type);

    Py_INCREF(&analysis_Type);
    PyModule_AddObject(m, "Analysis", (PyObject *)&analysis_Type);
}

#endif
