#ifdef HAVE_CONFIG_H
#ifdef _WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif
//#if defined(HAS_PYTHON)
//#include <Python.h>
//#endif
#include <string>
#include <sstream>
#include <cstdlib>
#include "AlibavaConfig.h"
#include "GtkUtils.h"
#include "PluginHook.h"
#include "ConfigFactory.h"
#include "utils.h"
#include "CalibTrimRun.h"
#include <iostream>

#include "AlibavaGUI.h"

void CalibrationGUI::user_init()
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();

    int2entry("txtStart", C.from());
    int2entry("txtEnd", C.to());
    int2entry("txtNpulse", C.npts());
    int2entry("txtSamples", C.nevts());
    int2entry("txtCalDelay", _cnf.delay());
    int2entry("txtCharge", _cnf.charge());

    if ( _cnf.scan_charge() )
        activate_button("btnScanVar", true );
    else
        activate_button("btnDelayScan", true );


    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("txtCalDelay"), 0 );
        gtk_widget_set_sensitive( get_widget("btnDelayScan"), 0 );
    }
    if (firmware() < 3)
    {
        gtk_widget_set_sensitive( get_widget("btnCalibTrim"), 0 );
    }
    g_signal_connect(G_OBJECT(get_widget("btnScanVar")),"toggled", G_CALLBACK(on_scan_changed),this);
}

void CalibrationGUI::process_text(const std::string &s, GtkWidget *w)
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    bool changed = false;
    if (s=="txtSamples")
    {
        C.nevts( entry2int(s) );
        changed = true;
    }
    else
    {
        if (s=="txtEnd")
        {
            int max = btn_active("btnScanVar") ? 261375 : 255 ;
            int val = entry2int(s);
            if (val>max)
                val = max;

            C.to( val );
            changed = true;
        }
        else if (s=="txtNpulse")
        {
            C.npts( entry2int(s) );
            if ( C.npts()>255 )
                C.npts(255);

            changed = true;
        }
        else if (s=="txtStart")
        {
            int val = entry2int(s, 0);
            if (val<0)
                val = 0;
            C.from( val );
            changed = true;
        }
        else if (s=="txtCalDelay")
        {
            _cnf.delay( entry2int(s) );
        }
        else if (s=="txtCharge")
        {
            _cnf.charge( entry2int(s) );
        }

    }
    if (changed)
    {
        C.adjust();
        if (_cnf.scan_charge())
            _cnf.charge_scan(C);
        else
            _cnf.delay_scan(C);

    }    check_values();

    C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    int2entry("txtStart", C.from());
    int2entry("txtEnd", C.to());
    int2entry("txtNpulse", C.npts());
    int2entry("txtSamples", C.nevts());
    int2entry("txtCalDelay", _cnf.delay());
    int2entry("txtCharge", _cnf.charge());
}

void CalibrationGUI::check_values()
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    C.to( entry2int("txtEnd") );
    C.from( entry2int("txtStart") );
    C.npts(entry2int("txtNpulse") );

    C.set_min_step( _cnf.scan_charge() ? 1024 : 1 );

    unsigned int max = _cnf.scan_charge() ? 261375 : 255 ;
    if (C.to() > max)
        C.to( max );

    if ( C.from() <= 0 )
        C.from(0);

    if ( C.npts()>255 )
        C.npts(255);

    C.adjust();

    if (_cnf.scan_charge())
        _cnf.charge_scan(C);
    else
        _cnf.delay_scan(C);
}

void CalibrationGUI::set_values()
{
    check_values();
    _cnf.charge( entry2int("txtCharge") );
    _cnf.delay( entry2int("txtCalDelay") );
    _cnf.scan_charge( btn_active("btnScanVar") );
}

void CalibrationGUI::on_scan_changed(GtkWidget *W, CalibrationGUI *cal)
{
    bool is_charge = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(W) );
    cal->_cnf.scan_charge(is_charge);

    ScanConfig C =  is_charge ? cal->_cnf.charge_scan() : cal->_cnf.delay_scan();
    cal->int2entry("txtStart", C.from());
    cal->int2entry("txtEnd", C.to());
    cal->int2entry("txtNpulse", C.npts());
    cal->int2entry("txtSamples", C.nevts());
}

void CalibrationGUI::auto_connect()
{
   // SET_CONNECT(on_scan_changed);
    SET_CONNECT(on_btn_calib_trim);
}

void CalibrationGUI::on_btn_calib_trim(GtkWidget *, CalibrationGUI *G)
{
	G->do_trim_calib = true;
    gtk_dialog_response(GTK_DIALOG(G->get_main_window()), GTK_RESPONSE_NONE);
}

void LaserSyncGUI::user_init()
{
    int2spin("spinFrom", _cnf.from());
    int2spin("spinTo", _cnf.to());
    int2spin("spinStep", _cnf.step());
    int2entry("txtSamples", _cnf.nevts());
}

void LaserSyncGUI::set_values()
{
    _cnf.from(spin2int("spinFrom") );
    _cnf.to(spin2int("spinTo") );
    _cnf.set_npts(spin2int("spinStep") );
    _cnf.nevts(entry2int("txtSamples") );
}

void BeetleConfigGUI::user_init()
{
    ConfigFactory *cf = ConfigFactory::instance();
    BeetleConfig::VarList::const_iterator ip;
    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        std::ostringstream ostr;
        ostr << "scale" << ip->first;
        const char *wname = ostr.str().c_str();
        GtkWidget *ww = get_widget(wname);
        GtkWidget *spn = get_widget(ip->first);
        if (ww)
        {
            gtk_range_set_adjustment(GTK_RANGE(ww),
                                     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spn)));

            gtk_spin_button_set_value(GTK_SPIN_BUTTON(spn), 0xff & ip->second);
            gtk_widget_queue_draw(ww);
        }
        else
            int2entry(ip->first, 0xff & ip->second);

        gtk_widget_queue_draw(spn);
    }
    activate_button("btnMaskChip0", _cnf.mask() & 1);
    activate_button("btnMaskChip1", _cnf.mask() & 2);

    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("btnMaskChip0"), 0 );
        gtk_widget_set_sensitive( get_widget("btnMaskChip1"), 0 );
    }

    if ( cf->easy() )
    {
//        gtk_widget_set_sensitive( get_widget("btnMaskChip0"), 0 );
//        gtk_widget_set_sensitive( get_widget("btnMaskChip1"), 0 );
        gtk_widget_hide( get_widget("frmMask"));
    }
}

struct bvalues {
        unsigned short addr, val;
        bvalues(unsigned short a=0, unsigned short b=0) : addr(a), val(b) {}
};

void BeetleConfigGUI::set_values()
{
    std::vector< bvalues > vval;
    BeetleConfig::VarList::const_iterator ip;

    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        unsigned short addr = (_cnf(ip->first) & 0xff00)>>8;
        unsigned short val  = (entry2int(ip->first) & 0xff);
        vval.push_back( bvalues(addr, val));

    }

    std::vector<bvalues>::iterator iv;
    for (iv=vval.begin(); iv!=vval.end();++iv)
    {
        _cnf.set(iv->addr, iv->val);
    }

    short mask = 0;
    if ( btn_active("btnMaskChip0") )
        mask |= 1;
    if ( btn_active("btnMaskChip1") )
        mask |= 2;

    _cnf.mask(mask);
}

void LaserConfigGUI::user_init()
{
    int2spin("spinDelay", _cnf.delay());
}

void LaserConfigGUI::set_values()
{
    _cnf.delay(spin2int("spinDelay"));
}

void TriggerConfigGUI::user_init()
{
    const char *spins[] = { "spinDACa", "spinDACb", "spinDACc", "spinDACd", 0 };
    const char *scales[] = { "scaleDACa", "scaleDACb", "scaleDACc", "scaleDACd", 0 };
    int i;
    for (i=0;spins[i]; i++)
    {
        gtk_range_set_adjustment(GTK_RANGE(get_widget(scales[i])),
                                 gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget(spins[i])))
                             );
    }
    click_button(_cnf.trigger_and() ? "rbtnAND" : "rbtnOR");
    click_button(_cnf.pulse() ? "rbtnTriggerPulse" : "rbtnTriggerIn");
    int2spin("spinDACa", _cnf.dacA() );
    int2spin("spinDACb", _cnf.dacB() );
    int2spin("spinDACc", _cnf.dacC() );
    int2spin("spinDACd", _cnf.dacD() );
}

void TriggerConfigGUI::set_values()
{
    _cnf.trigger_and(btn_active("rbtnAND") );
    _cnf.pulse(btn_active("rbtnTriggerPulse") );
    _cnf.dacA(spin2int("spinDACa") );
    _cnf.dacB(spin2int("spinDACb") );
    _cnf.dacC(spin2int("spinDACc") );
    _cnf.dacD(spin2int("spinDACd") );
}

void DAQConfigGUI::user_init()
{
    int2entry("txtMonChannel", _cnf.chan());
    int2entry("txtSampleSize", _cnf.sample_size());
    int2entry("entryMaxEvents", _cnf.nevts());
    double2entry("txtDelay", _cnf.delay());
    if (firmware()<2)
    {
        gtk_widget_set_sensitive( get_widget("btnEnableBusy"), 0);
        gtk_widget_set_sensitive( get_widget("btnPulseShape"), 0);
    }
    else
    {
        activate_button("btnPulseShape", _cnf.pulse_shape());
        activate_button("btnEnableBusy", _cnf.enable_busy());
    }
}

void DAQConfigGUI::set_values()
{
    _cnf.chan(entry2int("txtMonChannel") );
    _cnf.sample_size(entry2int("txtSampleSize") );
    _cnf.nevts( entry2int("entryMaxEvents"));
    _cnf.delay( entry2double("txtDelay"));
    _cnf.enable_busy( btn_active("btnEnableBusy") );
    _cnf.pulse_shape( btn_active("btnPulseShape"));
}

void DAQConfigGUI::process_text(const std::string &s, GtkWidget *w)
{
    if (s=="entryMaxEvents")
    {
        int uval = 0;
        if ( s.empty() )
        {
            uval = G_MAXINT;
        }
        else
        {
            uval = entry2int(s, G_MAXINT);
        }
        _cnf.nevts(uval);
    }
}


void AnalysisConfigGUI::user_init()
{
    double2entry("txtPedWeight", _cnf.ped_weight());
    double2entry("txtNoiseWeight", _cnf.noise_weight());
    double2entry("txtSeedCut", _cnf.seed_cut());
    double2entry("txtNeighCut", _cnf.neigh_cut());
    double2entry("txtSNCut", _cnf.sn_cut());
    click_button(_cnf.do_cmmd() ? "btnCmmdActive" : "btnCmmdInactive");
    if ( _cnf.polarity() == 0.0)
        click_button("btnXXPolarity");
    else
    {
        if (_cnf.polarity()>0.0)
            click_button("btnPosPolarity");
        else
            click_button("btnNegPolarity");
    }
}



void AnalysisConfigGUI::on_activate_common_mode(GtkWidget *w, AnalysisConfigGUI *A)
{

}

void AnalysisConfigGUI::auto_connect()
{
    SET_CONNECT(on_activate_common_mode);
}

void AnalysisConfigGUI::set_values()
{
    _cnf.set_weights(entry2double("txtPedWeight"), entry2double("txtNoiseWeight"));
    _cnf.seed_cut(entry2double("txtSeedCut"));
    _cnf.neigh_cut(entry2double("txtNeighCut"));
    _cnf.sn_cut(entry2double("txtSNCut"));
    _cnf.do_cmmd(btn_active("btnCmmdActive"));

    double polarity = 0.0;
    if ( btn_active("btnXXPolarity") )
        polarity = 0.0;
    else
    {
        if ( btn_active("btnPosPolarity"))
            polarity = 1.0;
        else
            polarity = -1.0;
    }
    _cnf.polarity( polarity );
}

void PluginConfigGUI::auto_connect()
{
    SET_CONNECT(on_browse_lib);
    SET_CONNECT(on_browse_path);
    SET_CONNECT(on_find_symbols);
}

void PluginConfigGUI::on_find_symbols(GtkWidget *, PluginConfigGUI *P)
{
    P->set_values();

    char *libname = P->_cnf.lib_path();
    PluginHook *ph = new PluginHook(libname);
    ph->init( P->get_xml_buffer(), P->get_buffer_size(), _("hook_dialog"));
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        gchar *dirname, *basename;
        const char *filename=ph->lib().c_str();
        const char *hook;
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        hook = ph->hook().c_str();
        P->_cnf.lib(basename);
        P->_cnf.path(dirname);
        P->_cnf.hook(hook);
        P->user_init();
        g_free(dirname);
        g_free(basename);
    }
    g_free(libname);
    delete ph;
}

void PluginConfigGUI::on_browse_lib(GtkWidget *, PluginConfigGUI *ps)
{
#ifdef G_OS_WIN32
    const char *filters[] = {"C/C++ libs|*.dll", "Python scripts|*.py","All|*", 0};
#elif defined(G_OS_UNIX)
    const char *filters[] = {"C/C++ (Linux) libs|*.so", "C/C++ (Mac OSX) libs|*.dylib", "Python scripts|*.py","All|*", 0};
#endif
    std::string ofile = ::get_file_name(_("Find shared library"), a_OpenFile, ps->_cnf.path().c_str(), filters);
    if (!ofile.empty())
    {
        gchar *dirname, *basename;
        const char *filename = ofile.c_str();
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        ps->set_text("txtLibName", basename);
        ps->set_text("txtLibPath", dirname);
        ps->_cnf.path(dirname);
        ps->_cnf.lib(basename);
        if (g_str_has_suffix(basename,".py"))
        {
            ps->_cnf.lang("python");
            ps->activate_button("btnPython", true);
        }
        else if  (g_str_has_suffix(basename,".so"))
        {
            ps->_cnf.lang("c++");
            ps->activate_button("btnCpp", true);
        }
        g_free(basename);
        g_free(dirname);
    }
}

void PluginConfigGUI::on_browse_path(GtkWidget *, PluginConfigGUI *ps)
{
    std::string ofile = ::get_file_name(_("Find shared library folder"), a_OpenFolder, 0, 0);
    if (!ofile.empty())
    {
        ps->set_text("txtLibPath", ofile);
        ps->_cnf.path(ofile);
    }
}

void PluginConfigGUI::user_init()
{
    set_text("txtLibPath", _cnf.path());
    set_text("txtLibName", _cnf.lib());
    set_text("txtHook", _cnf.hook());
    if ( _cnf.lang() == "python" )
    {
        activate_button("btnPython", true);
    }
    else
        activate_button("btnCpp", true);
}

void PluginConfigGUI::set_values()
{
    _cnf.path( get_text("txtLibPath") );
    _cnf.lib( get_text("txtLibName") );
    _cnf.hook( get_text("txtHook"));
    if ( btn_active("btnPython") )
    {
        _cnf.lang( "python" );
    }
    else
    {
        _cnf.lang("c++");
    }
}

ComparatorConfigGUI::ComparatorConfigGUI(ComparatorConfig &C, int fw)
    : ConfigGUI(fw), _cnf(C)
{
}

void ComparatorConfigGUI::auto_connect()
{
    SET_CONNECT(on_enable);
    SET_CONNECT(btn_mask_all);
    SET_CONNECT(btn_clear_all);
    SET_CONNECT(btn_reset_trim);
    SET_CONNECT(on_new_chip);
}

void ComparatorConfigGUI::user_init()
{
    GtkWidget *btn = get_widget("toggleComparator");
    if (_cnf.isEnabled())
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Disable Comparator");
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Enable Comparator");
    }
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("tableComparator"), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("notebookComparator"), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("tableThrs"), _cnf.isEnabled() );


    gtk_combo_box_set_active(GTK_COMBO_BOX(get_widget("chipCombo")), 0);
    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleThrs")),
                                     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinThrs"))));

    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleIcomp")),
                                         gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinIcomp"))));


    GtkAdjustment *adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinIcomp")));
    gtk_signal_connect(GTK_OBJECT(adj),
                       "value-changed",
                       GTK_SIGNAL_FUNC(icomp_changed), this);


    adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinThrs")));
    gtk_signal_connect(GTK_OBJECT(adj),
                       "value-changed",
                       GTK_SIGNAL_FUNC(thrs_changed), this);

    gtk_signal_connect(GTK_OBJECT(get_widget("btnNeg")),
                           "toggled",
                           GTK_SIGNAL_FUNC(polarity_changed), this);


    bool is_track = _cnf.getMode()==ComparatorConfig::trackMode;
    if (is_track)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnTrack")),1);
    else
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnPulse")),1);

    on_new_chip(get_widget("comboChip"), this);

}

void ComparatorConfigGUI::polarity_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    int iPol = C->btn_active("btnNeg") ? 1 : 0;
    C->_cnf.setPolarity(ichip, iPol);

    //std::cout << "ComparatorConfigGUI: Chip " << ichip << " changes polarity to " << iPol << std::endl;
}

void ComparatorConfigGUI::thrs_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    double val = gtk_adjustment_get_value(GTK_ADJUSTMENT(w));
    C->_cnf.setMainTh(ichip, (unsigned short)val);
}
void ComparatorConfigGUI::icomp_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    double val = gtk_adjustment_get_value(GTK_ADJUSTMENT(w));
    C->_cnf.setIcomp(ichip, (unsigned short)val);
}


void ComparatorConfigGUI::on_enable(GtkWidget *W, ComparatorConfigGUI *C)
{
    GtkWidget *btn = C->get_widget("toggleComparator");
    if (C->btn_active("toggleComparator"))
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Disable Comparator");
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Enable Comparator");
    }
    C->_cnf.setEnable( C->btn_active("toggleComparator") );
    gtk_widget_set_sensitive( C->get_widget("tableComparator"), C->_cnf.isEnabled() );
    gtk_widget_set_sensitive( C->get_widget("notebookComparator"), C->_cnf.isEnabled() );
    gtk_widget_set_sensitive( C->get_widget("tableThrs"), C->_cnf.isEnabled() );
}

void ComparatorConfigGUI::set_values()
{
    _cnf.setEnable( btn_active("toggleComparator") );

    ComparatorConfig::CompMode iMode = btn_active("btnTrack") ? ComparatorConfig::trackMode : ComparatorConfig::pulseMode;
    _cnf.setMode(iMode);

}

void ComparatorConfigGUI::process_text(const std::string &s, GtkWidget *w)
{
    double val;
    GtkWidget *cmb = get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));


    if (s == "txtDelta")
    {
        val = entry2double(s, 0.0);
        if (val < 0.0)
        {
            double2entry(s, 0.0);
            val = 0.0;
        }
        else if (val > 255)
        {
            double2entry(s, 255);
            val = 255.0;
        }
        else
        {
            //val = int(val/7.8125e-3)*7.8125e-3;
            //double2entry(s, val);
        }
        _cnf.setDeltaTh(ichip, val);
    }
    else if (s == "txtVrs")
    {
        val = entry2double(s, 0.0);
        if (val < 0.0)
        {
            double2entry(s, 0.0);
            val = 0.0;
        }
        else if (val > 255)
        {
            double2entry(s, 255);
            val = 255;
        }
        else
        {
            //val = int(val/0.0048828125)*0.0048828125;
            //double2entry(s, val);
        }
        _cnf.setVrc(ichip, val);
    }
}

void ComparatorConfigGUI::btn_mask_all(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setMask(ichip, j, true);

    on_new_chip(cmb, C);
}

void ComparatorConfigGUI::btn_clear_all(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setMask(ichip, j, false);

    on_new_chip(cmb, C);
}

void ComparatorConfigGUI::on_new_chip(GtkWidget *w, ComparatorConfigGUI *C)
{
    int ichip;
    if ( w )
        ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
    else
        ichip = 0;

    GtkWidget *btn;
    int iPol = C->_cnf.getPolarity(ichip);
    if (iPol)
        btn = C->get_widget("btnNeg");
    else
        btn = C->get_widget("btnPos");

    g_object_set(G_OBJECT(btn),"active", 1, NULL);
    gtk_widget_queue_draw(btn);    

    //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(C->get_widget("btnPos")), iPol);
    //std::cout << "ComparatorConfigGUI: New chip " << ichip << " selected. Polarity is " << iPol << std::endl;

    C->double2entry("txtDelta", C->_cnf.getDeltaTh(ichip));
    C->double2entry("txtVrs", C->_cnf.getVrc(ichip));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(C->get_widget("spinThrs")), C->_cnf.getMainTh(ichip));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(C->get_widget("spinIcomp")), C->_cnf.getIcomp(ichip));



    GtkTable *tbl;

    tbl = GTK_TABLE(C->get_widget("tableMask"));
    C->draw_table( tbl, ichip, 0 );

    tbl = GTK_TABLE(C->get_widget("tableTrim"));
    C->draw_table( tbl, ichip, 1 );

}

void ComparatorConfigGUI::draw_table(GtkTable *tbl, int ichip, int k)
{

    GtkWidget *T = GTK_WIDGET(tbl);
    // Now the heavy part... Resize the tables with the masks and test bits
    int i, ic, ir;
    int nchan = 128;
    int ncol = 16;
    int nrow = nchan / ncol + ( (nchan % ncol) ? 1 : 0);
    GtkWidget *tmp;
    char line[10];


    gtk_container_foreach(GTK_CONTAINER(T),
                          (GtkCallback) gtk_widget_destroy,
                          NULL);
    gtk_table_resize(tbl, nrow, ncol + 1);
    for (i = 0, ic = 0, ir = -1; i < nchan; i++, ic++)
    {
        if (!(i % ncol))
        {
            ir++;
            ic = 0;
            // add the label
            sprintf(line, "%d", i);
            tmp = gtk_label_new(line);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
            ic++;
        }
        if (k == 0)
        {
            // create the check button
            tmp = gtk_check_button_new();
            gtk_widget_show(tmp);
            gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(tmp), FALSE);
            gtk_widget_set_size_request(tmp, 10, 10);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND), (GtkAttachOptions) (0), 0, 0);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), _cnf.getMask(ichip, i));
            gtk_signal_connect(GTK_OBJECT(tmp), "toggled",
                               GTK_SIGNAL_FUNC(btn_toggled), this);

        }
        else
        {
            sprintf(line, "%d", _cnf.getTrim(ichip, i));
            tmp = gtk_entry_new();
            gtk_entry_set_width_chars(GTK_ENTRY(tmp),3);
            gtk_entry_set_text(GTK_ENTRY(tmp), line);
            gtk_widget_set_size_request(tmp, 20, 20);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND |GTK_FILL),
                             (GtkAttachOptions) (0), 0, 0);
            gtk_signal_connect(GTK_OBJECT(tmp), "changed",
                               GTK_SIGNAL_FUNC(txt_changed), this);

        }
        gtk_object_set_data(GTK_OBJECT(tmp), "chip", (gpointer) (intptr_t)ichip);
        gtk_object_set_data(GTK_OBJECT(tmp), "channel", (gpointer) (intptr_t)i);
        gtk_object_set_data(GTK_OBJECT(tmp), "type", (gpointer) (intptr_t)k);
    }
    gtk_widget_queue_resize(T);



}

void ComparatorConfigGUI::btn_toggled(GtkWidget *w, ComparatorConfigGUI *C)
{
    intptr_t ichip = intptr_t(gtk_object_get_data(GTK_OBJECT(w), "chip"));
    intptr_t ichan = intptr_t(gtk_object_get_data(GTK_OBJECT(w), "channel"));
    bool lval   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));

    C->_cnf.setMask(ichip, ichan, lval);
}

void ComparatorConfigGUI::txt_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    intptr_t ichip = intptr_t(gtk_object_get_data(GTK_OBJECT(w), "chip"));
    intptr_t ichan = intptr_t(gtk_object_get_data(GTK_OBJECT(w), "channel"));
    const char *cval = gtk_entry_get_text(GTK_ENTRY(w));

    char *endp;
    int ival = strtol(cval, &endp, 0);
    if (endp != cval )
    {
        ival &= 0x1f;
        C->_cnf.setTrim(ichip, ichan, ival);
    }
}

void ComparatorConfigGUI::btn_reset_trim(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setTrim(ichip, j, 0);

    on_new_chip(cmb, C);
}




static gboolean change_daq_button(ComparatorConfigTrimGUI *A)
{
    GtkWidget *w = A->get_widget("btnStartTrim");
    if (w)
    {
        gtk_button_set_label(GTK_BUTTON(w),
                             (A->running() ? _("Running (click to stop)") : _("Start Trim Calib.")));
        gtk_widget_show_all(w);
        gtk_widget_queue_draw(w);
    }


    int active = A->running() ? FALSE : TRUE;
    w = A->get_widget("btnOK");
    gtk_widget_set_sensitive(w, active);
    w = A->get_widget("btnCancel");
    gtk_widget_set_sensitive(w, active);

    A->set_idle(-1);
    return FALSE;
}


struct ConfigTrimPriv
{
	GtkWidget *hstGain;
	GtkWidget *hstTrim;
	CalibTrimRun *cr;
	GThread *thrd;

	ConfigTrimPriv() : hstGain(0), hstTrim(0), cr(0), thrd(0) {}
};
ComparatorConfigTrimGUI::ComparatorConfigTrimGUI(ComparatorConfig &C, int fw)
	: ConfigGUI(fw), _cnf(C), _idle(-1), _running(false)
{
	_priv = new ConfigTrimPriv;
}
ComparatorConfigTrimGUI::~ComparatorConfigTrimGUI()
{
    if (_priv->thrd)
        g_thread_join(_priv->thrd);

    if (get_idle()>0)
    {
        g_source_remove(get_idle());
        set_idle(-1);
    }

	delete _priv;
}
void ComparatorConfigTrimGUI::auto_connect()
{
    SET_CONNECT(on_button_start);
    SET_CONNECT(on_btn_clicked);
    SET_CONNECT(on_compute_trim);
    SET_CONNECT(on_out_folder);
    SET_CONNECT(on_save_data);
}

void ComparatorConfigTrimGUI::set_handlers()
{
    handlers["create_histograms"] = (GladeHandler)create_histograms;
}

static
void calib_trim_start_scan(Emitter *em, int val, CalibTrimRun *cr)
{
    /**
     * The Scan is about to start.
     * Set the trigger threshold to proper values
     */
	//info_msg(_("Starting Trim scan"));
    cr->get_board()->trigger(Alibava::TriggerPulse,
                             Alibava::TriggerOR,
                             -200 + 2048, -200 + 2048,
                             -100 + 2048, 100 + 2048);

}

static
void calib_trim_finished_scan(Emitter *em, int val, CalibTrimRun *cr)
{
    //info_msg(_("Trim scan finished"));
    ConfigFactory *cfg = ConfigFactory::instance();
    cfg->config_trigger( *cr->get_board() );
}

void ComparatorConfigTrimGUI::user_init()
{
    GtkWidget *W;
    ConfigFactory *cfg = ConfigFactory::instance();
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));

    W = get_widget("chipCombo");
    int wchip = 0;
    switch (beetle->mask() )
    {
        case 3:
            gtk_combo_box_set_active(GTK_COMBO_BOX(W), 0);
            break;
        case 1:
            gtk_combo_box_set_active(GTK_COMBO_BOX(W), 1);
            break;
        case 2:
        	wchip = 1;
            gtk_combo_box_set_active(GTK_COMBO_BOX(W), 2);
            break;
    }
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnNeg")), _cnf.getPolarity(wchip));

    if ( cfg->easy() )
    {
        gtk_widget_set_sensitive(W, FALSE);
    }

    gtk_signal_connect(GTK_OBJECT(get_widget("chipCombo")),
        				   "changed",
    					   GTK_SIGNAL_FUNC(on_new_chip), this);

    gtk_signal_connect(GTK_OBJECT(get_widget("btnNeg")),
    				   "toggled",
					   GTK_SIGNAL_FUNC(polarity_changed), this);

    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleTargetThrs")),
                                     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinTargetThrs"))));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinTargetThrs")), 6*1024);

    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleIcomp")),
                                             gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinIcomp"))));

    ComparatorConfig &comp = beetle->get_comparator_settings();
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinIcomp")), comp.getIcomp(wchip));


    int2entry("txtNevts", 100);
    int2entry("txtDeltaTrim", _cnf.getDeltaTh(0));
	activate_button("btnPulse", _cnf.getMode()==ComparatorConfig::pulseMode);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnSpread") ), false);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnFastScan") ), true);
   //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnDoTrim") ), false);
   //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnIthMain") ), false);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnCompGain") ), false);

   if ( cfg->easy() )
   {
	   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( get_widget("btnPos") ), true );
   }

   int nevts  = entry2int("txtNevts", 50);
   int target = gtk_spin_button_get_value(GTK_SPIN_BUTTON(get_widget("spinTargetThrs")));

   CalibTrimRun *cr = new CalibTrimRun(DAQmanager::get_the_manager()->get_board(),
                                       &_cnf,
                                       target,
                                       nevts, beetle->mask(),
                                       _priv->hstGain, _priv->hstTrim);
   cr->set_gui(this);
   _priv->cr = cr;
   g_signal_connect(cr->getEmitter(), "start", G_CALLBACK(calib_trim_start_scan), cr);
   g_signal_connect(cr->getEmitter(), "finished", G_CALLBACK(calib_trim_finished_scan), cr);

   // int2entry("txdtFixTrim", cr->get_fix_trim_value());
   // activate_button("btnFixTrim", cr->fix_trim());

   /*
    * Note: for now we remove the detailed scan and just do the trim from noise
    */
   //gtk_widget_hide(get_widget("btnDoTrim"));
   //gtk_widget_hide(get_widget("btnIthMain"));
   //gtk_widget_hide(get_widget("btnComputeTrim"));
   gtk_widget_queue_resize(get_main_window());

}

void ComparatorConfigTrimGUI::polarity_changed(GtkWidget *w, ComparatorConfigTrimGUI *C)
{
	GtkWidget *cmb = C->get_widget("chipCombo");
	int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
	int iPol = C->btn_active("btnNeg") ? 1 : 0;
	int ichip0=0, ichip1=1;
	switch (ichip)
	{
		case 1:
			ichip1=0;
			break;
		case 2:
			ichip0=1;
			break;
	}
	for (ichip=ichip0; ichip<=ichip1; ichip++)
	{
		//std::cout << "ComparatorConfigTrimGUI. Chip " << ichip << " changes Pol. to " << iPol << std::endl;	
		C->_cnf.setPolarity(ichip, iPol);
	}
}

void ComparatorConfigTrimGUI::on_new_chip(GtkWidget *w, ComparatorConfigTrimGUI *C)
{
    int ichip;
    if ( w )
        ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
    else
        ichip = 0;

    int wchip =0;
    switch (ichip)
    {
        case 1:
            wchip=0;
            break;
        case 2:
            wchip=1;
            break;
    }

    GtkWidget *btn;
    int iPol = C->_cnf.getPolarity(wchip);
    if (iPol)
        btn = C->get_widget("btnNeg");
    else
        btn = C->get_widget("btnPos");

    g_object_set(G_OBJECT(btn),"active", 1, NULL);
    gtk_widget_queue_draw(btn);    

    //std::cout << "ComparatorConfigTrimGUI: chip " << wchip << " selected. Pol. is " << iPol << std::endl;

}


void ComparatorConfigTrimGUI::stop()
{
    if (running())
    {
        _priv->cr->stop();
        _running = false;
    }
    if (_priv->thrd)
        g_thread_join(_priv->thrd);

}

void ComparatorConfigTrimGUI::start()
{
    if (!running() && _priv->cr )
    {
    	_running = true;
        _priv->cr->start_calib();
    }
}

GtkWidget * ComparatorConfigTrimGUI::create_histograms(ComparatorConfigTrimGUI *wnd, char *name, char *s1, char *s2,
                                         int i1, int i2)
{
	std::string hnam(name);
	HstHistogram2D *hh;
	GtkWidget      *hst2d = 0;

	if (hnam=="hstCompGain")
	{
		hh = hst_histogram2d_new (5, -0.5, 4.5, 128, -0.5, 255.5, 0);
		hst2d = hst_chart2d_new ( HST_HISTOGRAM2D(hh));
		wnd->_priv->hstGain = hst2d;
	}
	else if (hnam=="hstTrimGain")
	{
		hh = hst_histogram2d_new (8, -0, 32, 64, 0.0, 256, 0);
		hst2d = hst_chart2d_new ( HST_HISTOGRAM2D(hh));
		wnd->_priv->hstTrim = hst2d;
	}
	if (hst2d)
	{
		hst_chart2d_set_squared(HST_CHART2D(hst2d),TRUE);
		hst_chart1d_monitor_start(HST_CHART1D(hst2d), 500);
	}
	return hst2d;
}
void ComparatorConfigTrimGUI::set_values()
{

}


gpointer calib_trim_thread(ComparatorConfigTrimGUI *C)
{
    C->start();
    C->stop();
    int rc = g_idle_add( (GSourceFunc) change_daq_button, C);
    C->set_idle(rc);
    return 0;
}

void ComparatorConfigTrimGUI::on_btn_clicked(GtkWidget *w, ComparatorConfigTrimGUI *C)
{
	if (C->running())
		C->stop();


	if ( C->get_idle()> 0 )
	{
	    g_source_remove( C->get_idle() );
	    C->set_idle(-1);
	}
}


void ComparatorConfigTrimGUI::on_save_data(GtkWidget *w, ComparatorConfigTrimGUI *C)
{
    C->_priv->cr->save_data(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
}

void ComparatorConfigTrimGUI::on_out_folder(GtkWidget *dialog, ComparatorConfigTrimGUI *C)
{
    std::string out_folder = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    C->_priv->cr->set_output_folder(out_folder);
}


void ComparatorConfigTrimGUI::on_compute_trim(GtkWidget *, ComparatorConfigTrimGUI *C)
{
	if (!C->_priv->cr)
		return;

	GtkWidget *dialog;
	GtkDialogFlags flags = (GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT);
	GtkWindow *wnd = GTK_WINDOW( C->get_main_window() );
	dialog = gtk_dialog_new_with_buttons( "Compute threshold Correction",
			    	                      wnd, flags,
										  GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE,
										  GTK_STOCK_OK, GTK_RESPONSE_OK,
										  GTK_STOCK_EXECUTE, GTK_RESPONSE_APPLY,
										  NULL);
	GtkBox    *area = GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG(dialog) ));
	GtkBox    *vbox = GTK_BOX(gtk_vbox_new(FALSE,1));
	GtkWidget *entry = gtk_entry_new();
	GtkWidget *result = gtk_label_new("");
	GtkWidget *help = gtk_label_new(_("Click on Execute to compute.\nClick on OK to save the results\nClick Quit to close the window"));
	gtk_label_set_justify(GTK_LABEL(result), GTK_JUSTIFY_LEFT);
	gtk_label_set_justify(GTK_LABEL(help), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(area, GTK_WIDGET(vbox), TRUE, TRUE,0);
	gtk_box_pack_start(vbox, gtk_label_new("Enter the value for Ithmain"), TRUE, TRUE, 0);
	gtk_box_pack_start(vbox, entry, TRUE, TRUE, 0);
	gtk_box_pack_start(vbox, result, TRUE, TRUE, 0);
	gtk_box_pack_start(vbox, help, TRUE, TRUE, 0);
	gtk_widget_show_all(GTK_WIDGET(dialog));
	int rc;
	while ( (rc=gtk_dialog_run(GTK_DIALOG(dialog))) != GTK_RESPONSE_CLOSE )
	{
		char line[512];
		bool save=false;
		double rms;
		int ithmain = atoi( gtk_entry_get_text(GTK_ENTRY(entry)));
		if (rc==GTK_RESPONSE_OK)
			save = true;

		rms = C->_priv->cr->compute_trim_corr(ithmain, save);
		if ( rms < 0)
		{
		    sprintf(line,_("Do not have Trim DAC fits.\nYou need to scan the trim DACs or load them from a file."));
		}
		else
		    sprintf(line,_("RMS for Ithmain %d is %.2f"), ithmain, rms);

		gtk_label_set_text(GTK_LABEL(result), line);
		gtk_widget_queue_draw(dialog);
	}
	gtk_widget_hide( dialog );
	gtk_widget_destroy(dialog);
}

void ComparatorConfigTrimGUI::on_button_start(GtkWidget *, ComparatorConfigTrimGUI *C)
{
    if (C->_running)
    {
        C->_priv->cr->stop();
        C->_running = false;
        int rc = g_idle_add( (GSourceFunc) change_daq_button, C);
        C->set_idle(rc);
        return;
    }
    int rc = g_idle_add( (GSourceFunc) change_daq_button, C);
    C->set_idle(rc);


    /*
     * Here comes the meat...
     */
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    int nevts = C->entry2int("txtNevts", 50);
    int target = gtk_spin_button_get_value(GTK_SPIN_BUTTON(C->get_widget("spinTargetThrs")));
    int idelta = C->entry2int("txtDeltaTrim");
    int icomp = gtk_spin_button_get_value(GTK_SPIN_BUTTON(C->get_widget("spinIcomp")));
    int mask = beetle->mask();
    int ichip0=0, ichip1=1;
    ComparatorConfig::CompMode iMode = C->btn_active("btnTrack") ? ComparatorConfig::trackMode : ComparatorConfig::pulseMode;

    if (ichip==2)
    {
        ichip0 = 1;
        mask = 2;
    }
    else if (ichip==1)
    {
        ichip1 = 0;
        mask = 1;
    }
    C->_cnf.setEnable(1);
    C->_cnf.setMode(iMode);
    for (ichip=ichip0; ichip<=ichip1; ichip++)
    {
        C->_cnf.setDeltaTh(ichip, idelta);
    }

//    CalibTrimRun *cr = new CalibTrimRun(the_daq_manager->get_board(),
//                                        &C->_cnf,
//                                        target,
//                                        nevts, mask,
//                                        C->_priv->hstGain, C->_priv->hstTrim);
//    cr->set_gui(C);
//    C->_priv->cr = cr;
//    g_signal_connect(cr->getEmitter(), "start", G_CALLBACK(calib_trim_start_scan), cr);
//    g_signal_connect(cr->getEmitter(), "finished", G_CALLBACK(calib_trim_finished_scan), cr);

    //bool do_fix_trim = C->btn_active("btnFixTrim");
    //int  fix_trim_val = C->entry2int("txtFixTrim", 20);
    //C->_priv->cr->set_fix_trim(do_fix_trim, fix_trim_val);
    C->_priv->cr->threshold( target );
    C->_priv->cr->mask( mask );
    C->_priv->cr->nevts(nevts);
    C->_priv->cr->icomp(icomp);
    C->_priv->cr->todo_list(   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON( C->get_widget("btnSpread"))),
                               gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(C->get_widget("btnFastScan"))),
                               false, false, false,
                               gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON( C->get_widget("btnCurrent"))),
                               gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON( C->get_widget("btnThrsScan"))));


#if GLIB_CHECK_VERSION(2,32,0)
    g_thread_new("CalibTrim",(GThreadFunc)calib_trim_thread, C);
#else
    GError *error=NULL;
    C->_priv->thrd = g_thread_create((GThreadFunc)calib_trim_thread, C, FALSE, &error);
    if (error)
    {
        err_msg("Could not start Trim Config Thread");
        C->_priv->thrd = 0;
        g_error_free(error);
    }
#endif
}

static char __sval__[128];
static gboolean set_ithmain_value(ComparatorConfigTrimGUI *A)
{
    GtkWidget *w = A->get_widget("lblInfo");
    gtk_label_set_text(GTK_LABEL(w), __sval__);
    return FALSE;
}
void ComparatorConfigTrimGUI::show_recomended_value(int val)
{
    sprintf(__sval__,"Recomended Ithmain value %d", val);
    g_idle_add( (GSourceFunc) set_ithmain_value, this);
}
