/*
 * AlibavaDataQueue.cc
 *
 *  Created on: Jun 20, 2014
 *      Author: lacasta
 */

#include "AlibavaDataQueue.h"

AlibavaDataQueue::AlibavaDataQueue()
{
	data_guard = g_mutex_new();
	data_mutex = g_mutex_new();
	condition = g_cond_new();
//	g_mutex_init(&data_guard);
//    g_mutex_init(&data_mutex);
//    g_cond_init(&condition);
}

AlibavaDataQueue::~AlibavaDataQueue()
{
    clear();
//    g_mutex_clear(&data_mutex);
//    g_mutex_clear(&data_guard);
//    g_cond_clear(&condition);
    g_mutex_free(data_guard);
    g_mutex_free(data_mutex);
    g_cond_free(condition);
}

void AlibavaDataQueue::clear()
{
    Guard G( data_mutex );
    while (!queue.empty())
    {
        AlibavaData *B = queue.front();
        queue.pop_front();
        delete B;
    }
}

void AlibavaDataQueue::push(AlibavaData *data)
{
    Guard G(data_mutex);
    queue.push_back(data);
    g_cond_broadcast(condition);
}

AlibavaData *AlibavaDataQueue::pop()
{
     Guard G(data_mutex);
    if (queue.empty())
        return 0;
    else
    {
        AlibavaData *data = queue.front();
        queue.pop_front();
        return data;
    }
}

bool AlibavaDataQueue::empty()
{
     Guard G(data_mutex);
    return queue.empty();
}

int AlibavaDataQueue::wait(int time_out)
{
    if (time_out<0)
    {
        g_mutex_lock (data_mutex);
        while (queue.empty())
            g_cond_wait (condition, data_mutex);
        g_mutex_unlock (data_mutex);
    }
    else
    {
        gint64 end_time =  g_get_monotonic_time () + 5 * G_TIME_SPAN_SECOND;
        g_mutex_lock (data_mutex);
        while (queue.empty())
        {
        	GTimeVal period;
        	g_get_current_time(&period);
        	g_time_val_add(&period, time_out*1000000);
        	if ( !g_cond_timed_wait(condition, data_mutex, &period) )
//            if (!g_cond_wait_until (condition, data_mutex, end_time))
            {
                // timeout has passed.
                g_mutex_unlock (data_mutex);
                return 1;
            }
        }
        g_mutex_unlock (data_mutex);
    }
    return 0;
}
