/* -*- mode: c++ -*- */
#ifndef __AlibavaGUI_h__
#define __AlibavaGUI_h__

#include <AlibavaConfig.h>
#include <GladeApp.h>
#include <Alibava.h>
#include <Analysis.h>
#include <DataFile.h>
#include <KeyFile.h>
#include <gtkhst/histogram.h>
#include <gtkhst/tracer.h>
#include <DAQmanager.h>
#include <string>
#include <deque>
#include <map>

struct AlibavaGUIPrivate;

class AlibavaGUI : public AlibavaGladeApp, public DAQmanager, public DataMonitor
{
    private:
        AlibavaGUIPrivate *priv;
        // The Alibava object
        // Alibava _board;

        // Gtk signal handlers
        static GtkWidget *create_histogram(AlibavaGUI *, char *name, char *s1, char *s2, int i1, int i2);

        static gboolean change_daq_button(AlibavaGUI *);
        static void on_open(GtkWidget *, AlibavaGUI *);
        static void on_save(GtkWidget *, AlibavaGUI *);
        static void on_save_as(GtkWidget *, AlibavaGUI *);
        static void on_save_pedestals(GtkWidget *, AlibavaGUI *);
        static void on_load_pedestals(GtkWidget *, AlibavaGUI *);
        static void on_buttonDAQ(GtkWidget *, AlibavaGUI *);
        static void on_about(GtkWidget *, AlibavaGUI *);
        static void on_runtype(GtkWidget *, AlibavaGUI *);
        static void on_reset(GtkWidget *, AlibavaGUI *);
        static void on_file_format(GtkWidget *, AlibavaGUI *);
        static void on_calibration_config(GtkWidget *, AlibavaGUI *);
        static void on_lasersync_config(GtkWidget *, AlibavaGUI *);
        static void on_beetle_config(GtkWidget *, AlibavaGUI *);
        static void on_laser_config(GtkWidget *, AlibavaGUI *);
        static void on_comparator_config(GtkWidget *, AlibavaGUI *);
        static void on_trigger_config(GtkWidget *, AlibavaGUI *);
        static void on_log_data(GtkWidget *, AlibavaGUI *);
        static void on_daq_config(GtkWidget *, AlibavaGUI *);
        static void on_analysis_config(GtkWidget *, AlibavaGUI *);
        static void on_data_type(GtkWidget *, AlibavaGUI *);
        static void on_signal_select(GtkWidget *, AlibavaGUI *);
        static void on_show_headers(GtkWidget *, AlibavaGUI *);
        static void on_time_profile(GtkWidget *, AlibavaGUI *);
        static void on_reset_hst(GtkWidget *, AlibavaGUI *);
        static void on_help_activate(GtkWidget *, AlibavaGUI *);
        static void on_plugin_activate(GtkWidget *, AlibavaGUI *);
        static void on_plugin_menu_activate(GtkWidget *, AlibavaGUI *);
        static void on_reset_pedestals(GtkWidget *, AlibavaGUI *);
        static void on_reset_hitmap(GtkWidget *, AlibavaGUI *);
        static void on_monitor_toggled(GtkWidget *, AlibavaGUI *);
        static void on_reconnect(GtkWidget *, AlibavaGUI *);
        static void on_latency_dialog(GtkWidget *, AlibavaGUI *);


        static gboolean shadow_widgets(AlibavaGUI *);
        static gboolean update_labels(AlibavaGUI *);
        static gboolean send_message(AlibavaGUI *);
        static gboolean send_b_config(AlibavaGUI *);
        static gboolean send_t_config(AlibavaGUI *);
        static gboolean send_c_config(AlibavaGUI *);
        static gboolean start_calib_trim(AlibavaGUI *);

        void reset_hst();

        gpointer _thread;
        HstHistogram *current_hst, *hitmap_hst, *event_hst;
        HstTracer *temp_hst, *time_hst, *noise_tracer[2], *cmmd_tracer[2];

        bool _show_time_profile, _do_monitor;
        int _cntxt;
        std::deque<std::string> _msg;
        std::string sfile;
        std::map<std::string, GtkWidget *> hst_list;

        void unref_histograms(GObject *);
        void create_time_profiles(GObject *, double, double);

        void set_run_manager( RunManager::RunType type);
        int check_event_headers(const AlibavaData* b);

        void fill_common_mode(Analysis* analysis);

    public:
        AlibavaGUI(int nevts, Alibava *B);
        ~AlibavaGUI();

        static void last_words(GtkWidget *, AlibavaGUI *);

        void lock();
        void unlock();

        // TODO: this is not elegant...
        static void on_btn_calib_trim(GtkWidget *, AlibavaGUI *);

        // GladeApp stuff
        void user_init();
        void set_handlers();
        void auto_connect();
        void process_text(const std::string &s, GtkWidget *w);

        std::string get_func_name();

        void device_opened();
        void config_loaded();
        int prepare_for_run();
        int stop_run();
        void process_event(const AlibavaData *);
        static gpointer acquire_thrd(AlibavaGUI *, bool is_thread=true);
        void acquire_thread(bool is_thread);


        Analysis &analysis();
        void nevts(int x);
        void sample_size(int x);

        void add_message(const char *msg);

        void create_signal_profile(const char *, int nb, double xmin, double xmax);
        void create_signal_histogram();

        void getHistogram(const std::string &whst, const std::string &type, size_t &size, char **buffer);
        void set_debug(bool x);
};

#endif
