/*
 * AlibavaMonitor.h
 *
 *  Created on: Jul 20, 2012
 *      Author: lacasta
 */

#ifndef ALIBAVAMONITOR_H_
#define ALIBAVAMONITOR_H_
#include <string>
#include <DataReceiver.h>
struct _GTimer;

struct AlibavaMonitorPriv;
class AlibavaMonitor : public DataReceiver
{
    private:
        GTimer *_timer;
        AlibavaMonitorPriv *_priv;
        std::string _time_elapsed;
        std::string _rate;
        std::string _run_type;
        std::string _out_file;

        int    _run_type_val;
        int    _ntrigger;
        int    _nexpected;
        int    _naccepted;
        double _eff;
        double _fraction;
        double _ratef;
        double _value;
        double _time;
        bool   _start_from_soap;

        double _ntrig0;
        double _tim0;
        void   *_data;
        bool _running;
    public:
        AlibavaMonitor();
        virtual ~AlibavaMonitor();
        double getFraction() const;
        int getNtrigger() const;
        const std::string getOutFile() const;
        const std::string getRate() const;
        double getRateAsNumber() const;
        void computeRate(int n_received);
        const std::string getTimeElapsed() const;
        double getTimeElapsedAsDouble() const;
        std::string computeTimeElapsed();
        void getHistogram(const std::string &whst, size_t &size, char **buffer);
        void getHistogram(const std::string &whst, const std::string &type, size_t &size, char **buffer);

        const std::string getRunType() const;
        int getRunTypeVal() const;
        void setRunTypeVal( int v );
        int getNexpected() const;
        void setData(void* data);
        void setFraction(double fraction);
        void setNtrigger(int ntrigger);
        void newTrigger();
        void setOutFile(const std::string& outFile);
        void setRate(const std::string& rate);
        void setRate(const double val);
        void setTimeElapsed(const std::string& timeElapsed);
        void setRunType(const std::string& runType);
        void setNexpected(int nexpected);
        double getValue() const;
        void setValue(double value);
        void from_soap(bool x) {_start_from_soap = x; }
        bool from_soap() const { return _start_from_soap; }
        void reset_counters();
        double getNtrig0() const;
        void setNtrig0(double ntrig0);
        double getTim0() const;
        void setTim0(double tim0);
        int getNaccepted() const;
        void setNaccepted(int naccepted);
        double getEfficiency() const;
        std::string getEfficiencyAsString(int set_prec=1) const;
        bool isRunning() const { return _running; }
        void isRunning(bool v);
        double getEff() const;
        void setEff(double eff);

        void receive(const AlibavaData *)
        {
            newTrigger();
        }
};
extern AlibavaMonitor __monitor__;

#endif /* ALIBAVAMONITOR_H_ */
