/*
 * BitSet.cc
 *
 *  Created on: Jun 19, 2013
 *      Author: lacasta
 */

#include <iostream>
#include <iomanip>
#include <cstring>
#include <cstdio>
#include "BitSet.h"

//---------------------------------------------------------------------
// Constructors & destructor
//---------------------------------------------------------------------
BitSet::BitSet(int nn, int szW) :
        pb(0)
{
    new_size(nn, szW);
}

BitSet::BitSet(int nn, unsigned char *ptr) :
        pb(0)
{
    int i, j;
    new_size(nn, 8 * sizeof(unsigned char)); //8 bits per byte; sizeof in bytes
    for (i = 0; i < nSlot; i++)
    {
        pb[i] = ptr[i];
        for (j = 0; j < szSlot; i++)
            if (pb[i] & (1 << j))
                nset++;
    }
}

BitSet::BitSet(const BitSet & b)
{
    nbits = b.nbits;
    szSlot = b.szSlot;
    nSlot = b.nSlot;
    nsTrue = nSlot;
    if (nSlot > 0)
    {
        pb = new unsigned char[nSlot];
        for (int i = 0; i < nSlot; i++)
            pb[i] = b.pb[i];
        nset = b.nset;
    }
}

BitSet::BitSet(int nn, char *sb)
{
    nsTrue = 0;
    szSlot = 8 * sizeof(unsigned char);
    BitStr(nn, sb);
}

BitSet::~BitSet()
{
    if (pb)
        delete[] pb;
}

void BitSet::new_size(int nn, int szW)
{
    if (pb)
        delete[] pb;
    nbits = nn;
    szSlot = szW;
    nSlot = nbits / szSlot + (nbits % szSlot ? 1 : 0);
    nsTrue = nSlot;
    nset = 0;
    if (nn == 0)
    {
        pb = 0;
        return;
    }
    pb = new unsigned char[nSlot];
    for (int i = 0; i < nSlot; i++)
        pb[i] = 0;
}

//---------------------------------------------------------------------
// Overloaded operators
//---------------------------------------------------------------------
int BitSet::operator[](int i) const
{
    return Bit(i);
}

BitSet *BitSet::Word(int x, unsigned char v)
{
    if (x >= 0 && x <= nsTrue)
        pb[x] = v;
    return this;
}

unsigned char BitSet::Word(int x) const
{
    if (x >= 0 && x < nsTrue)
        return pb[x];
    else
        return 0;
}

BitSet *BitSet::operator !()
{
    // set all elements to 0
    return AllZero();
}

BitSet *BitSet::AllZero()
{
    // set all elements to 0
    for (int i = 0; i < nSlot; i++)
        pb[i] = 0;
    nset = 0;
    return this;
}

BitSet & BitSet::operator =(const BitSet & b)
{
    nbits = b.nbits;
    szSlot = b.szSlot;
    if (b.nSlot > nsTrue)
    {
        delete[] pb;
        pb = new unsigned char[b.nSlot];
        nsTrue = b.nsTrue;
    }
    nSlot = b.nSlot;
    for (int i = 0; i < nSlot; i++)
        pb[i] = b.pb[i];
    nset = b.nset;

    return *this;
}

//---------------------------------------------------------------------
// Interface
//---------------------------------------------------------------------
BitSet *BitSet::Reset(unsigned char *ptr)
{
    int i, j;

    for (i = 0, nset = 0; i < nSlot; i++)
    {
        pb[i] = ptr[i];
        for (j = 0; j < szSlot; j++)
            if (pb[i] & (1 << j))
                nset++;
    }
    return this;
}

void BitSet::BitStr(int nn, const char *sb)
{
    int i, j, k, iok;
    const char *p, *q;
    char c=0;
    int slts = nn / szSlot + (nn % szSlot ? 1 : 0);

    if (slts > nsTrue)
    {
        delete[] pb;
        pb = new unsigned char[slts];
        nsTrue = slts;
    }
    nSlot = slts;
    nbits = nn;
    for (i = 0; i < nSlot; i++)
        pb[i] = 0;

    // Set the bits
    for (p = sb, i = 0, c = '0'; *p; c = *p++)
    {
        switch (*p)
        {
            case '1':
                Set(i++);
                break;
            case '0':
                Clr(i++);
                break;
            case '<':
                q = strchr(p, '>');
                iok = sscanf(p, "<%d>", &j);
                if (iok != 1)
                {
                    printf("BitSet::BitSet(string): bad format: %s\n", sb);
                    return;
                }
                if (c == '1')
                    for (k = 0; k < j; k++)
                        Set(i++);
                else
                    for (k = 0; k < j; k++)
                        Clr(i++);
                p = q;
                break;
            default:
                i++;
                break;
        }
    }
}

BitSet *BitSet::Set(int i)
{
    int j = i % szSlot, k = i / szSlot;
    pb[k] |= (1L << j);
    nset++;
    return this;
}

BitSet *BitSet::Set(int val, int start, int width)
{
    //   int j= start%szSlot,k=start/szSlot;
    for (int i = start, l = 0; l < width && i < nbits; l++, i++)
        if (val & (1 << l))
            Set(i);
        else
            Clr(i);

    return this;
}

BitSet *BitSet::Clr(int i)
{
    int j = i % szSlot, k = i / szSlot;
    if ((pb[k] & (1L << j)) != 0)
        nset--;
    pb[k] &= ~(1L << j);
    return this;
}

unsigned long BitSet::Byte(int start, int width) const
{
    unsigned short rc = 0;
    width = (width > nbits ? nbits : width);

    for (int i = start, j = 0; j < width && i < nbits; j++, i++)
        if (Bit(i))
            rc |= (1 << j);

    return rc;
}

int BitSet::Bit(int i) const
{
    int j = i % szSlot, k = i / szSlot;
    return (pb[k] & (1L << j)) != 0;
}

//---------------------------------------------------------------------
// I/O operations
//---------------------------------------------------------------------
void BitSet::Print(int start, int width)
{
    int i, j;
    width = (width > nbits ? nbits : width);
    for (i = start, j = 0; j < width && i < nbits; i++, j++)
        if (Bit(i))
            std::cout << '1';
        else
            std::cout << '0';

    std::cout << std::endl;
}

void BitSet::prt(std::ostream & os)
{
    int i;
    os << "Number of bits " << nbits << std::endl;
    for (i = 0; i < nbits; i++)
    {
        if (Bit(i))
        {
            os << '1';
        }
        else
        {
            os << '0';
        }
    }
}

std::ostream & operator <<(std::ostream & os, BitSet & b)
{
    b.prt(os);
    return os;
}

void BitSet::Dump()
{
    std::cout << "Number of bits : " << nbits << std::endl << "Number of slots: " << nSlot
            << std::endl << "Size of slot     " << szSlot << " bits " << std::endl;
    for (int i = 0; i < nSlot; i++)
#ifdef WIN32
        std::cout << std::setw(8) << std::hex << pb[i];
#else
        std::cout << std::setw(8) << std::setbase(16) << pb[i];
#endif

    std::cout << std::dec << std::endl;
}

int BitSet::byte_reverse(int val, int nbit) const
{
    int ib, rc = 0, nb = nbit - 1;
    for (ib = 0; ib <= nb; ib++)
    {
        if (val & (1 << ib))
        {
            rc |= (1 << (nb - ib));
        }
    }
    return rc;
}

