/*
 * CalibrationRun.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */

#include <iostream>
#include "DAQmanager.h"
#include "CalibrationRun.h"
#include "Analysis.h"
#include "DataMonitor.h"
#include "ScanConfig.h"
#include <ConfigFactory.h>
#include "utils.h"

CalibrationRun::CalibrationRun(DAQmanager *D)
    : RunManager(D)
{

}

CalibrationRun::~CalibrationRun()
{
}

void CalibrationRun::start_of_run()
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    DAQmanager *daqm = get_daq_manager();
    DataFile *data_file = daqm->out_file();
    ConfigFactory *cfg = ConfigFactory::instance();
    Analysis *ana = dynamic_cast<Analysis *>(cfg->get("analysis"));
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
    ana->update_pedestals(false);

    ScanConfig C = cal->scan_charge() ? cal->charge_scan() : cal->delay_scan();
    int scan_type = cal->scan_charge() ? 1 : 2;
    DataMonitor *mon = daqm->get_monitor();
    if (mon)
        mon->create_signal_profile("injected pulse",
                                   C.npts(),
                                   C.from(),
                                   C.to() );
    daqm->set_expected( C.npts()*C.nevts() );
    daqm->set_received(0);
}

int CalibrationRun::loop( std::ostream &oheader)
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    unsigned int rc, i, step;
    ConfigFactory *cfg = ConfigFactory::instance();
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
    ScanConfig C = cal->scan_charge() ? cal->charge_scan() : cal->delay_scan();
    DAQmanager *daqm = get_daq_manager();
    DataFile *data_file = daqm->out_file();

    int scan_type = cal->scan_charge() ? 1 : 2;
    if (data_file)
        data_file->scan_definition(scan_type, cal->charge(), cal->delay(), C.npts(), C.from(), C.to(), C.nevts());

    step = C.step();
    oheader << C.npts() << "; "
            << C.from() << "; "
            << C.to()  << "; "
            << C.step();
    if ( ! cal->scan_charge() )
        oheader << "; delay_scan";

    rc = 0;
    unsigned int ntrg;
    alibava->define_scan(C.nevts(), C.npts(), C.from(), C.to());
    for (i=0; i<C.npts();i++)
    {
        int start = daqm->get_received();
        alibava->reset_trigger_counter();

        if (daqm->stop())
            break;

        if ( (rc=acquire(C.from()+ i *step)) )
            return rc;

        if ( (rc=read_data()) )
            return rc;

        ntrg = alibava->get_trigger_counter();
        incNtrig(ntrg);

        double val = C.from() + i*step;
        if (data_file)
        	data_file->new_point(start, daqm->get_received(), val);
        alibava->end_of_scan_point(val, ntrg);

    }
    return 0;
}

int CalibrationRun::acquire(unsigned int val)
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    int rc = 0;
    ConfigFactory *cfg = ConfigFactory::instance();
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
    ScanConfig C = cal->scan_charge() ? cal->charge_scan() : cal->delay_scan();
    if ( cal->scan_charge() )
        rc = get_daq_manager()->get_board()->calibration(val, cal->delay(), C.nevts());
    else
        rc = get_daq_manager()->get_board()->calibration(cal->charge(), val, C.nevts());

    return rc;
}



ChargeScanRun::ChargeScanRun(DAQmanager *D)
    : RunManager(D)
{

}

void ChargeScanRun::start_of_run()
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
}

int ChargeScanRun::loop( std::ostream &oheader)
{
    Alibava *alibava = get_daq_manager()->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    int rc;
    ConfigFactory *cfg = ConfigFactory::instance();
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
    ScanConfig C = cal->charge_scan();

    rc = alibava->charge_scan_run(cal->delay(), C.nevts(), C.npts(), C.from(), C.to());

    return rc;
}
