/* -*- mode: c++ -*- */
#ifndef __DAQConfig_h__
#define __DAQConfig_h__

#include <ConfigFile.h>

/** This class contains the parameters for the DAQ
 *  It has 2 parameters:
 *  -chan: is the channel that will be monitorized
 *  -nevt_per_sample: When in RadSource mode, we will
 *                    acquire events in chuncks of 
 *                    nevt_per_sample 
 * 
 */
class DAQConfig : public ConfigFile
{
    private:
        int _chan;
        int _nevt_per_sample;
        int _nevts;
        double _delay;
        unsigned short _fw_config;

        void cpy(const DAQConfig &c);
    public:
        DAQConfig();
        DAQConfig(const DAQConfig &c);
        ~DAQConfig();

        DAQConfig &operator=(const DAQConfig &b);
        void load_data();
        void save_data();

        int chan() const;
        void chan(int x);

        int sample_size() const;
        void sample_size(int x);
        int nevts() const;

        void nevts(int x);
        double delay() const;
        void delay(double x);

        unsigned short firmware_config() const;
        void pulse_shape(bool x);
        bool pulse_shape() const;

        void enable_busy(bool x);
        bool enable_busy() const;
};

#endif
