/* -*- mode: c++ -*- */
#ifndef __DataFile_h__
#define __DataFile_h__

#include <string>
#include <DataFormat.h>

/**
 * class DataFile
 *
 * Defines a Generc DataFile object.
 * This is the object in charge of storing the data.
 * The default one does nothing
 */
class DataFile
{
    public:
        enum BlockType { NewFile=0, StartOfRun, DataBlock, CheckPoint, EndOfRun, UserDataBlock=0x1000 };
        enum FileType { AsciiFile, HDF5, UnknownFile };

	private:
        std::string _fnam;
        std::string _header;

    public:
        DataFile(const std::string &name="output.dat") :
            _fnam(name)
        {
        }
        virtual ~DataFile()
        {
        }

        const std::string &file_name() const
        {
            return _fnam;
        }
        void set_file_name(const std::string &f) { _fnam = f; }


        // Opens the file
        virtual void open(const std::string &nam="")
        {
        }
        // Closes the file
        virtual void close()
        {
        }

        /**
         * Describes the UserData format
         */
        virtual void set_data_format(const char *fmt) {}

        // Writes the current event
        virtual void write(const AlibavaData *evt)
        {
        }

        // Writes the current value of pedestals and noise
        // Also stores the run type, given in type
        virtual void write(int type, double *p, double *n)
        {
        }

        /**
         *  Writes a buffer in a given block.
         *  It is used to write the user defined data when using plugins
         */
        virtual void write( BlockType, size_t size, const char *buff)
        {

        }

        /*
         * A string with extra information that will used
         * in different ways in different implementations
         */
        void set_header(const std::string &s)
        {
            _header = s;
        }

        const std::string &get_header() const { return _header; }

        /**
         * Receives information about a scan
         */
        virtual void scan_definition(int type, int time, int delay, int npoints, int from, int to, int nevts) {}

        /**
         * Receives the setup information
         */
        virtual void describe_setup(int run_type, int nchip, int chip_mask, int firmware) {}

        /**
         * Receives a the information about a new point
         */
        virtual void new_point(int start, int end, double value) {}

        /**
         * Flushes to file
         */
        virtual void flush() {}

        /**
         * The factory method.
         * This method should be able to detect the file type
         * and return the proper implementation of DataFile to
         * handle the file.
         */
        static DataFile *factory(FileType type, const std::string fnam);
};

#endif
