#if HAVE_CONFIG_H
#ifdef _WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif
//#include "PythonLoader.h"
#include "PluginHook.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <vector>
#include "GtkUtils.h"
#include  "SearchPath.h"
#include "string_utils.h"
#include "utils.h"

#include <vector>
#include <string>
typedef std::vector<std::string> SymbolList;
int DumpFile(char * filename, SymbolList &s_list);


int find_functions_in_module(const char *module_name, std::vector<std::string> &vout);
#ifdef HAS_PYTHON
void find_python_symbols(const gchar *module, std::vector< std::string> &symb);
#endif

extern gchar *__daemon_path__;

//static void
//ss_child_watch( GPid  pid,
//                gint  status,
//                gpointer data )
//{
//
//    /* Close pid */
//    g_spawn_close_pid( pid );
//}

int find_symbols(const std::string &fnam, std::vector<std::string> &v)
{
    int rc;
    static char line[256];
    gchar *cmd;

    std::ostringstream ocmd;
    FILE *pf;
    v.clear();
#ifdef G_OS_UNIX
#ifdef __APPLE__
    cmd = g_strdup_printf("nm -g  %s | grep ' T _[a-zA-Z]' | grep -v '(' | cut -d ' ' -f 3  | sort", fnam.c_str());
#else
    cmd = g_strdup_printf("nm -C --defined  %s | grep ' T ' | grep -v '(' | cut -d ' ' -f 3  | sort", fnam.c_str());
#endif
    pf = popen(cmd,"r");
    while ( fgets(line,256, pf) )
    {
        if ( strstr(line,"no symbol") )
        {
            // no symbol
            break;
        }
        else if ( strstr(line,"_fini") || strstr(line,"_init"))
        {
            continue;
        }

        std::string ss = trim_str(line);
        ss = ss.substr( ss.find_last_of(' ')+1);
#ifdef __APPLE__
        ss.erase(ss.begin());
#endif
        v.push_back( ss );
    }
    rc = pclose(pf);

#elif defined( G_OS_WIN32 )
    cmd = 0;

    SymbolList s_list;
    DumpFile( (char *)fnam.c_str(), s_list);

    SymbolList::iterator it;
    for (it=s_list.begin(); it!=s_list.end(); it++)
    {
        std::string ss = trim_str(*it);
        ss = ss.substr( ss.find_last_of(' ')+1);
//        size_t p = ss.find('@');
//        if (p!=ss.npos)
//            ss[p] = '\0';
        v.push_back( ss );
    }
#else
    return 0;
#endif


    g_free(cmd);
    return rc;
}


PluginHook::PluginHook(const std::string &L, const std::string &H) :
    _lib(L), _hook(H)
{
}

PluginHook::~PluginHook()
{
}

void PluginHook::auto_connect()
{
    SET_CONNECT(on_help);
    SET_CONNECT(on_browse_lib);
    SET_CONNECT(on_find_symbols);
    SET_CONNECT(on_hook_selected);
}

void PluginHook::user_init()
{
    if (!_lib.empty())
    {
        set_text("entryLibrary", _lib);
        on_find_symbols(0, this);
    }
}

void PluginHook::on_hook_selected(GtkWidget *w, PluginHook *ph)
{
    GtkWidget *active = gtk_menu_get_active(GTK_MENU(w));
    ph->_hook = gtk_widget_get_name(active);
}

void PluginHook::on_find_symbols(GtkWidget *w, PluginHook *ph)
{
    if (ph->_lib.empty())
        return;

    std::vector<std::string> symb;
    std::vector<std::string>::const_iterator ip;
    std::string the_lib = ph->get_text("entryLibrary");
    if (the_lib.empty())
        return;

    GtkWidget *optmenu = ph->get_widget("comboHook");
    gtk_option_menu_remove_menu(GTK_OPTION_MENU(optmenu));

    SearchPath P;
    const char *ldpath = getenv("LD_LIBRARY_PATH");
    if (ldpath)
        P.add_path(ldpath);
    the_lib = P.find_file(the_lib);

#ifdef HAS_PYTHON
    if (g_str_has_suffix(the_lib.c_str(),".py"))
    {
        find_python_symbols(the_lib.c_str(), symb);
    }
    else
#endif
        find_symbols(the_lib, symb);


    if ( !symb.empty() )
    {
        GtkWidget *menu = gtk_menu_new();

        for (ip=symb.begin(); ip!=symb.end();ip++)
        {
            GtkWidget *item = gtk_menu_item_new_with_label(ip->c_str());
            gtk_widget_set_name(item, (const gchar *)ip->c_str());
            gtk_widget_show(item);
            gtk_menu_append(GTK_MENU(menu), item);
        }
        gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
        g_signal_connect(G_OBJECT(menu),"selection_done",G_CALLBACK(on_hook_selected), ph);
        gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu),0);
    }
    else
    {
        GtkWidget *menu = gtk_menu_new();
        GtkWidget *item = gtk_menu_item_new_with_label("");
        gtk_widget_show(item);
        gtk_menu_append(GTK_MENU(menu), item);
        gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
        g_signal_connect(G_OBJECT(menu),"selection_done",G_CALLBACK(on_hook_selected), ph);
        gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu),0);
    }
}

void PluginHook::on_browse_lib(GtkWidget *w, PluginHook *P)
{
    std::string ofile = ::get_file_name("Find shared library", a_OpenFile, 0, 0);
    if (!ofile.empty())
    {
        P->_lib = ofile;
        P->set_text("entryLibrary", ofile);
    }
}

void PluginHook::on_help(GtkWidget *, PluginHook *P)
{

}

void PluginHook::process_text(const std::string &s, GtkWidget *w)
{
    std::string val = get_text(s);
    if (s=="entryLibrary")
        _lib = get_text(s);
    else
        _hook = get_text(s);
}
