#include <iostream>
#include <sstream>
#include <cmath>
#include "ScanConfig.h"

void ScanConfig::cpy(const ScanConfig &c)
{
    set_group(c.get_group() );
    _from = c._from;
    _to = c._to;
    _npts = c._npts;
    _nevts= c._nevts;
    _min_step = c._min_step;
    adjust();
}

void ScanConfig::adjust()
{
    unsigned int tmp;
    if (_npts == 0)
        _npts = 1;

    if (_from > _to)
    {
        tmp = _from;
        _from = _to;
        _to = tmp;
    }
    double step = double(_to - _from)/double(_npts);
    
    // We stick to multiples of the minimum step
    double nstp = rint(step/_min_step);
    if (nstp==0)
        nstp = 1;
    
    step = nstp*_min_step;
    
    // now adjust the beginning of the range
    _from = int(rint(double(_from)/_min_step)*_min_step);
    _to   = int(_from + step*_npts);
    
}

void ScanConfig::load_data()
{
    if (exists())
    {
        from(get_int("from") );
        to(get_int("to") );
        npts(get_int("npts") );
        nevts(get_int("nevts") );


    }
}

void ScanConfig::save_data()
{
    set_int("from", from());
    set_int("to", to());
    set_int("npts", npts());
    set_int("nevts", nevts());
}


CalibrationConfig::CalibrationConfig(unsigned int charge, unsigned short delay) :
     _delay(delay), _charge(charge), _scan_charge(true),
    _charge_scan(32, 0, 30720, 1024), _delay_scan(32, 0, 255, 1)
{
    set_group("calibration");
    _charge_scan.set_group("charge_scan");
    _delay_scan.set_group("delay_scan");
}


CalibrationConfig::CalibrationConfig(const CalibrationConfig &c)
{
    set_group("calibration");
    _charge_scan.set_group("charge_scan");
    _delay_scan.set_group("delay_scan");

    _delay = c._delay;
    _charge = c._charge;
    _scan_charge = c._scan_charge;
    _charge_scan = c._charge_scan;
    _delay_scan = c._delay_scan;
}
CalibrationConfig &CalibrationConfig::operator=(const CalibrationConfig &c)
{
    if (&c != this)
    {
        _delay = c._delay;
        _charge = c._charge;
        _scan_charge = c._scan_charge;
        _charge_scan = c._charge_scan;
        _delay_scan = c._delay_scan;
    }
    return *this;
}

void CalibrationConfig::load_data()
{
    if (_charge_scan.key_file() != key_file())
        _charge_scan.set_file(key_file());
    else
        _charge_scan.load_data();

    if (_delay_scan.key_file() != key_file())
        _delay_scan.set_file(key_file());
    else
        _delay_scan.load_data();

    if (exists())
    {
        int dd = get_int("delay", 127);
        delay(dd);

        dd = get_int("charge", 100000);
        charge(dd);

        bool which = get_bool("scan_charge",true);
        scan_charge(which);


    }
}

void CalibrationConfig::save_data()
{
    set_bool("scan_charge", scan_charge());
    set_int("delay", delay());
    set_int("charge", charge());
    _charge_scan.save_data();
    _delay_scan.save_data();
}
