//#define DEBUG 1
#include "USBFifo.h"
#include "utils.h"

USBFifo::USBFifo(USBport &p)
    : fifo(259000), port(p)
{
}

USBFifo::~USBFifo()
{
}

int USBFifo::open(const std::string &fnam)
{
    int rc = port.open(fnam);
    if (rc)
        return rc;
    
    (&port)->setup_fifo(&fifo);

    return 0;
}

int USBFifo::close()
{
    fifo.detach();
    return port.close();
}

void USBFifo::resize(unsigned int sz)
{
    fifo.resize(sz);
}

int USBFifo::write(void *buf, int size)
{
    return port.write(buf,size);
}


#define CHUNK_SIZE 10240
int USBFifo::read(void *buf, int size)
{
    int rc;
    if ( fifo.running())
        rc = fifo.read((char *)buf, size);
    else
    {
        rc =  port.read((char *)buf, size);
    }
    return rc;
}

int USBFifo::flush()
{
    int rc = 0;

    if ( fifo.running() )
        fifo.flush();
    else
        port.flush();

    return rc;
}

int USBFifo::wait_for_data(double tmout)
{
    if (fifo.running())
    {
        PDEBUG("fifo waiting for data\n");
        fifo.wait(tmout);
        return fifo.data_available();
    }
    else
        return port.wait_for_data(tmout);
}

unsigned int USBFifo::data_available(unsigned int sz)
{
    if (fifo.running())
        return fifo.data_available();
    else
        return port.data_available(sz);
}

int USBFifo::poll(int timeout)
{
    if ( fifo.running())
    {
        PDEBUG("USBFifo poll");
        bool rc = fifo.wait( timeout/1000. );
        return rc ? 0 : 1;
    }
    else
        return port.poll(timeout);
}
