/*
 * USBserialwin32.cc
 *
 *  Created on: Jan 11, 2012
 *      Author: lacasta
 */
#include <glib.h>
#ifdef HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#include <iostream>
#include <vector>
#include <unistd.h>
#include "Fifo.h"
#include "USBserialWnds.h"
#ifdef G_OS_WIN32
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#ifdef HAS_FTD2XX
#include <d2xx/ftd2xx.h>
#endif
#else
#include "d2xx/WinTypes.h"
#endif
#include "utils.h"

struct USBserial_priv
{
    int fd;
    HANDLE hfd;
    HANDLE hThreadTerm;
};

std::vector<USBserial_priv *> handle_list;

HANDLE get_handle_from_fd(int fd)
{
    return handle_list[fd]->hfd;
}
HANDLE get_thread_stopper_from_fd(int fd)
{
    return handle_list[fd]->hThreadTerm;
}

USBserialWnds::USBserialWnds()
    : fd(0), _priv(0)
{
    _priv = g_new0(struct USBserial_priv, 1);
}

USBserialWnds::USBserialWnds(const char *fnam)
    : USBport(fnam), fd(0), _priv(0)
{
    _priv = g_new0(struct USBserial_priv, 1);
    open(fnam);
}


USBserialWnds::~USBserialWnds() {
    g_free(_priv);
}

#ifdef G_OS_UNIX
int USBserialWnds::open(const std::string &fnam)
{
    return 0;
}
int USBserialWnds::close()
{
    return 0;
}
int USBserialWnds::read(void *buf, int size)
{
    return 0;
}
int USBserialWnds::write(void *buf, int size)
{
    return 0;
}
int USBserialWnds::flush()
{
    return 0;
}

int USBserialWnds::poll(int timeout)
{
    return 0;
}

int USBserialWnds::fildes() const
{
    return fd;
}
int USBserialWnds::setup_fifo(Fifo *)
{
    return 0;
}

void USBserialWnds::stop_thread()
{

}

unsigned int USBserialWnds::data_available(unsigned int size) { return 0; }
int USBserialWnds::wait_for_data(double tmout) { return 0; }

#else

#ifdef HAS_FTD2XX
LONG __connect__()
{
    FT_HANDLE fthandle;
    FT_STATUS ftstatus;
    DWORD numdevs=0;
    DWORD flags=0;
    DWORD locID=0;
    DWORD deviceID=0;
    DWORD type=0;
    DWORD ftdevice=0;
    LONG port_number=-1;
    char serialnumber[16];
    char description[64];
    DWORD deviceindex;
    unsigned int br = 115200, rd=5000, wr=5000;

    ftstatus = FT_CreateDeviceInfoList(&numdevs);
    if (ftstatus != FT_OK) {
        return -1;
    }
    if (numdevs > 0 ) {
        for (DWORD i = 0; i < numdevs; i++) {
            ftstatus = FT_GetDeviceInfoDetail(i,&flags,&type,&deviceID,&locID,
            		                          serialnumber,description,&fthandle);
            if (ftstatus == FT_OK) {
                if (deviceID == 0x04036001) {
                    deviceindex = i;
                    ftstatus = FT_Open(deviceindex, &fthandle);
                    ftstatus = FT_SetDataCharacteristics(fthandle,
                    		FT_BITS_8,
                    		FT_STOP_BITS_1,
                    		FT_PARITY_NONE);
                    ftstatus = FT_SetBaudRate(fthandle, br);
                    ftstatus = FT_SetTimeouts(fthandle, rd, wr);
                    ftstatus = FT_SetLatencyTimer(fthandle, 2);
                    ftstatus = FT_GetComPortNumber(fthandle, &port_number);
                    ftstatus = FT_Close(fthandle);

                    break;
                }
            }
            else {
                break;
            }
        }
        if (ftstatus == FT_OK) {
            return port_number;
        }
        else {
            return -1;
        }
    }
    else {
        return -1;
    }
}
#endif



int USBserialWnds::open(const std::string &fnam)
{
    char line[64];
    char comports[16][10]={"\\\\.\\COM1",  "\\\\.\\COM2",  "\\\\.\\COM3",  "\\\\.\\COM4",
                           "\\\\.\\COM5",  "\\\\.\\COM6",  "\\\\.\\COM7",  "\\\\.\\COM8",
                           "\\\\.\\COM9",  "\\\\.\\COM10", "\\\\.\\COM11", "\\\\.\\COM12",
                           "\\\\.\\COM13", "\\\\.\\COM14", "\\\\.\\COM15", "\\\\.\\COM16"};
    int i, j=-1;
    gchar *inam = g_strdup( fnam.c_str() );
    int port_num = -1;
#ifdef HAS_FTD2XX
    if ( ( port_num =  __connect__() ) < 0 )
    {
    	err_msg(_("Apparently there is no Alibava board connected"));
    	g_free(inam);
    	return -1;
    }
    if ( port_num > 0 )
    {
    	sprintf(line,"COM%d", port_num);
    	g_free(inam);
    	inam = g_strdup(line);
    }
#endif
    for (i=0; i<16;i++)
    {
        sprintf(line,"COM%d", i+1);
        if ( !strcmp(line, inam) )
        {
            j=i;
            break;
        }
    }
    if (j<0)
    {
        err_msg(_("Port %s does not exist"), inam);
        g_free(inam);
        return -1;
    }
    g_free(inam);

    _priv->hfd = CreateFile(comports[j],
    		GENERIC_READ|GENERIC_WRITE,
            0,                          /* no share  */
            NULL,                       /* no security */
            OPEN_EXISTING,
#ifdef WNDS_OVERLAPPED
            FILE_FLAG_OVERLAPPED,        // we want overlapped operation
#else
            0,
#endif
            NULL);                      /* no templates */

    if(_priv->hfd == INVALID_HANDLE_VALUE )
    {
        err_msg(_("unable to open port %s"), line);
        return -1;
    }

    DCB port_settings;
    const char *baudr ="baud=9600 data=8 parity=N stop=1";

    memset(&port_settings, 0, sizeof(port_settings));  /* clear the new struct  */
    port_settings.DCBlength = sizeof(port_settings);

    if( !BuildCommDCBA(baudr, &port_settings))
    {
      err_msg(_("unable to set com port dcb settings"));
      CloseHandle(_priv->hfd);
      _priv->hfd = 0;
      return(1);
    }
    // Common settings
    port_settings.fOutxCtsFlow = FALSE;
    port_settings.fOutxDsrFlow = FALSE;
    port_settings.fDtrControl = DTR_CONTROL_DISABLE;
    port_settings.fDtrControl = FALSE;
    port_settings.fDsrSensitivity = FALSE;
    port_settings.fOutX = FALSE;
    port_settings.fInX = FALSE;
    port_settings.fNull = FALSE;
    port_settings.fRtsControl = RTS_CONTROL_DISABLE;
    port_settings.fAbortOnError = FALSE;

    if(!SetCommState(_priv->hfd, &port_settings))
    {
      err_msg(_("unable to set com port cfg settings"));
      CloseHandle(_priv->hfd);
      _priv->hfd = 0;
      return(1);
    }

    COMMTIMEOUTS Cptimeouts;
    GetCommTimeouts (_priv->hfd, &Cptimeouts);

/**
    Cptimeouts.ReadIntervalTimeout         = MAXDWORD;
    Cptimeouts.ReadTotalTimeoutMultiplier  = 0;
    Cptimeouts.ReadTotalTimeoutConstant    = 0;
    Cptimeouts.WriteTotalTimeoutMultiplier = 0;
    Cptimeouts.WriteTotalTimeoutConstant   = 0;
*/
    Cptimeouts.ReadIntervalTimeout         = 5000;
    Cptimeouts.ReadTotalTimeoutMultiplier  = 100;
    Cptimeouts.ReadTotalTimeoutConstant    = 1000;
    Cptimeouts.WriteTotalTimeoutMultiplier = 10;
    Cptimeouts.WriteTotalTimeoutConstant   = 100;


    if(!SetCommTimeouts(_priv->hfd, &Cptimeouts))
    {
      err_msg(_("unable to set com port time-out settings"));
      CloseHandle(_priv->hfd);
      return(1);
    }

#ifdef WNDS_OVERLAPPED
    SetCommMask(_priv->hfd, EV_RXCHAR|EV_TXEMPTY);
#endif

    fd = _priv->fd = handle_list.size();
    _priv->hThreadTerm = CreateEvent(0,0,0,0);
    handle_list.push_back(_priv);

    return 0;
}
int USBserialWnds::close()
{
    CloseHandle(_priv->hfd);
    CloseHandle(_priv->hThreadTerm);
    handle_list[_priv->fd] = 0;
    return 0;
}

int USBserialWnds::read(void *buf, int size)
{
#ifdef WNDS_OVERLAPPED
	DWORD dwRead;
	BOOL fWaitingOnRead = FALSE;
	OVERLAPPED osReader = {0};
	char lastError[1024];
	TCHAR *err;
	int rc;

	// Create the overlapped event. Must be closed before exiting
	// to avoid a handle leak.
	osReader.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if (!fWaitingOnRead)
	{
		// Issue read operation.
		if (!(rc=ReadFile(_priv->hfd, buf, size, &dwRead, &osReader)))
		{
			if (GetLastError() != ERROR_IO_PENDING)
			{	  // read not delayed?
				// Error in communications; report it.
				FormatMessage(
						FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
						NULL,
						rc,
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						(LPTSTR)&err,
						0,
						NULL);

				err_msg("USBSerialWnds::read [rc=%d (%d, %d)] -> %s", rc, rc, dwRead, err);

				LocalFree(err);
				return -1;
			}
			else
			{
				fWaitingOnRead = TRUE;
			}
		}
		else {
			// read completed immediately
			CloseHandle(osReader.hEvent);
			return dwRead;
		}
	}

	DWORD dwRes;
	rc = 0;
	if (fWaitingOnRead)
	{
	   dwRes = WaitForSingleObject(osReader.hEvent, INFINITE);
	   switch(dwRes)
	   {
	      // Read completed.
	      case WAIT_OBJECT_0:
	          if (!GetOverlappedResult(_priv->hfd, &osReader, &dwRead, FALSE))
	          {
	        	  // Error in communications; report it.
	        	  FormatMessage(
	        			  FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
						  NULL,
						  rc,
						  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						  (LPTSTR)&err,
						  0,
						  NULL);

	        	  err_msg("USBSerialWnds::read [rc=%d (%d, %d)] -> %s", rc, dwRes, dwRead, err);
	        	  LocalFree(err);
	        	  rc = -1;
	          }
	          else
	             rc = dwRead;

	          //  Reset flag so that another opertion can be issued.
	          fWaitingOnRead = FALSE;
	          break;

	      case WAIT_TIMEOUT:
	          // Operation isn't complete yet. fWaitingOnRead flag isn't
	          // changed since I'll loop back around, and I don't want
	          // to issue another read until the first one finishes.
	          //
	          // This is a good time to do some background work.
	          rc = -3;
	          break;

	      default:
	          // Error in the WaitForSingleObject; abort.
	          // This indicates a problem with the OVERLAPPED structure's
	          // event handle.
	          rc = -1;
	   }
	}
	CloseHandle(osReader.hEvent);
	return rc;
#else
	int n=0, itry=0;
	TCHAR *err;
    DWORD error;
    BOOL rc;
	while (itry<100)
	{
		rc = ReadFile(_priv->hfd, buf, size, (LPDWORD)((void *)&n), 0);
		error = GetLastError();
		if ( rc && n)
			return n;

		if ( rc && error == 0)
		{
			itry++;
		}
		else
			break;
	}

    char lastError[1024];
    FormatMessage(
    		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
            NULL,
            error,
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR)&err,
            0,
            NULL);

    err_msg("USBSerialWnds::read [rc=%d (%d, %d)] -> %s", error, rc, n, err);
    LocalFree(err);
    return -1;
        
#endif
}
int USBserialWnds::write(void *buf, int size)
{
#ifdef WNDS_OVERLAPPED
	OVERLAPPED osWrite = {0};
	DWORD dwWritten;
	DWORD dwRes;
	BOOL fRes;

	// Create this write operation's OVERLAPPED structure's hEvent.
	osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (osWrite.hEvent == NULL)
		// error creating overlapped event handle
		return -1;

	// Issue write.
	if (!WriteFile(_priv->hfd, buf, size, &dwWritten, &osWrite))
	{
		if (GetLastError() != ERROR_IO_PENDING) {
			// WriteFile failed, but isn't delayed. Report error and abort.
			fRes = FALSE;
		}
		else
		{
			// Write is pending.
			dwRes = WaitForSingleObject(osWrite.hEvent, INFINITE);

			switch(dwRes)
			{
			// OVERLAPPED structure's event has been signaled.
			case WAIT_OBJECT_0:
				if (!GetOverlappedResult(_priv->hfd, &osWrite, &dwWritten, FALSE))
					fRes = FALSE;
				else
					// Write operation completed successfully.
					fRes = TRUE;
				break;

			default:
				// An error has occurred in WaitForSingleObject.
				// This usually indicates a problem with the
				// OVERLAPPED structure's event handle.
				fRes = FALSE;
				break;
			}
		}
	}
	else
		// WriteFile completed immediately.
		fRes = TRUE;

	CloseHandle(osWrite.hEvent);
	return fRes ? dwWritten : -1;
#else
	BOOL iRet = false ;
	DWORD dwBytesWritten = 0;
	DWORD error;
	iRet = WriteFile (_priv->hfd, buf, size, &dwBytesWritten,0);
	if ( iRet )
		return dwBytesWritten;

	char lastError[1024];
	FormatMessage(
			FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			error,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			lastError,
			1024,
			NULL);
	err_msg("%s", lastError);
	return 0;

#endif
}

int USBserialWnds::flush()
{
    return 0;
}

/**
 * Polls for new data arriving.
 * Timeout: if <0 wait for ever
 *          if >0 wait for timeout milliseconds
 *
 *  Returns: 0  if there is data
 *           >0 if the call gave a timeout
 *           <0 an error
 */

//#define __usame__

int USBserialWnds::poll(int tmout)
{
#ifndef WNDS_OVERLAPPED
	return 0;
#else
	int rc = -1;
	DWORD      dwRes, dwOvRes;
	DWORD      dwCommEvent;
	DWORD      dwStoredFlags;
	BOOL      fWaitingOnStat = FALSE;
	OVERLAPPED osStatus = {0};

	dwStoredFlags = EV_RXCHAR;
	if (!SetCommMask(_priv->hfd, EV_RXCHAR))
		// error setting communications mask; abort
		return -1;

	osStatus.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	if (osStatus.hEvent == NULL)
		// error creating event; abort
		return -1;

	if (tmout<0)
		tmout = INFINITE;

	printf("Calling WaitCommEvent\n");
	if (!WaitCommEvent(_priv->hfd, &dwCommEvent, &osStatus))
	{
		if (GetLastError() != ERROR_IO_PENDING)
		{
			// error in WaitCommEvent; abort
			char lastError[1024];
			FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL,
					GetLastError(),
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
					lastError,
					1024,
					NULL);
			err_msg("USBserianWnds::poll %s", lastError);
			//CloseHandle(osStatus.hEvent);
			return -1;
		}
	}

	// Wait a little while for an event to occur.
	printf("Calling WaitForSingleObject with time out: %d\n", tmout);
	dwRes = WaitForSingleObject(osStatus.hEvent, tmout);
	printf("after WaitForSingleObject: %d %d\n", tmout, dwRes);
	switch(dwRes)
	{
	// Event occurred.
	case WAIT_OBJECT_0:
		if (!GetOverlappedResult(_priv->hfd, &osStatus, &dwOvRes, FALSE))
		{
			char lastError[1024];
			FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL,
					GetLastError(),
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
					lastError,
					1024,
					NULL);
			err_msg("USBserialWnds::poll [1] %s", lastError);

			CloseHandle(osStatus.hEvent);
			return -1;
		}
		else
		{
			if ( dwCommEvent & EV_RXCHAR)
			{
				rc = 0;
			}
			else
			{
				char lastError[1024];
				FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						lastError,
						1024,
						NULL);
				err_msg("USBserialWnds::poll [2] %s", lastError);
				CloseHandle(osStatus.hEvent);
				return -1;
			}
			ResetEvent(osStatus.hEvent);
		}
		break;

	case WAIT_TIMEOUT:
		// Operation isn't complete yet.
		rc = 1;
		break;

	default:
		// Error in the WaitForSingleObject; abort
		// This indicates a problem with the OVERLAPPED structure's
		// event handle.
		{
			char lastError[1024];
			FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL,
					GetLastError(),
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
					lastError,
					1024,
					NULL);
			err_msg("USBserialWnds::poll [2] %s", lastError);
			CloseHandle(osStatus.hEvent);
			return -1;
		}
	}
	printf("poll: output is %d\n", rc);
	CloseHandle(osStatus.hEvent);
	return rc;

#endif
}

int USBserialWnds::fildes() const
{
    return fd;
}
int USBserialWnds::setup_fifo(Fifo *F)
{
    return F->attach(this);
}

void USBserialWnds::stop_thread()
{
    SetEvent(_priv->hThreadTerm);
}

void USBserialWnds::wait_for_port_data(Fifo *F)
{
    bool abContinue = true;
    DWORD dwEventMask=0;
    char szTmp[1024];
    int iSize  = sizeof ( szTmp );

    OVERLAPPED ov;
    memset(&ov,0,sizeof(ov));
    ov.hEvent = CreateEvent( 0,true,0,0);
    HANDLE arHandles[2];
    arHandles[0] = _priv->hThreadTerm;

    DWORD dwWait;
    while (  !F->endpoll() && abContinue )
    {

        BOOL abRet = WaitCommEvent(_priv->hfd,&dwEventMask, &ov) ;
        if ( !abRet )
        {
            if( GetLastError () != ERROR_IO_PENDING)
            {
            	char lastError[1024];
            	    FormatMessage(
            	            FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
            	            NULL,
            	            GetLastError(),
            	            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            	            lastError,
            	            1024,
            	            NULL);
            	    err_msg("%s", lastError);
            	    return;
            }
        }


        arHandles[1] = ov.hEvent ;

        dwWait = WaitForMultipleObjects (2,arHandles,FALSE,INFINITE);
        switch ( dwWait )
        {
            case WAIT_OBJECT_0:
                break;

            case WAIT_OBJECT_0 + 1:
            {
                DWORD dwMask;
                if (GetCommMask(_priv->hfd,&dwMask) )
                {
                    if ( dwMask == EV_TXEMPTY )
                    {
                        ResetEvent ( ov.hEvent );
                        continue;
                    }

                }

                //read data here...
                try
                {
                    BOOL abRet = false;

                    DWORD dwBytesRead = 0;
                    OVERLAPPED ovRead;
                    ovRead.hEvent = CreateEvent( 0,true,0,0);

                    do
                    {
                        ResetEvent( ovRead.hEvent  );
                        memset(szTmp,0,sizeof szTmp);
                        abRet = ReadFile(_priv->hfd,szTmp,sizeof(szTmp),&dwBytesRead,&ovRead);
                        if (!abRet )
                        {
                            abContinue = FALSE;
                            break;
                        }
                        if ( dwBytesRead > 0 )
                        {
                            F->write(szTmp, dwBytesRead, true);
                        }
                    } while ( dwBytesRead > 0 );
                    CloseHandle(ovRead.hEvent );
                }
                catch(...)
                {
                	char lastError[1024];
                	FormatMessage(
                			FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                			NULL,
                			GetLastError(),
                			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                			lastError,
                			1024,
                			NULL);
                	err_msg(_("Exception in thread\n%s"), lastError);
                	return;
                }

                ResetEvent ( ov.hEvent );
            }
            break;
        }//switch
    }
    return;
}

unsigned int USBserialWnds::data_available(unsigned int size)
{
	COMSTAT status;
	DWORD errors;
	ClearCommError(_priv->hfd, &errors, &status);

	return status.cbInQue;
}
int USBserialWnds::wait_for_data(double tmout)
{
#ifdef ExperiMental
	DWORD dwWait;
	DWORD dwEventMask=0;
	OVERLAPPED ov;
	memset(&ov,0,sizeof(ov));

	ov.hEvent = CreateEvent( 0,true,0,0);
	if (tmout<0)
		tmout = 1.e20;
	while (true)
	{
		BOOL abRet = WaitCommEvent(_priv->hfd,&dwEventMask, &ov) ;
		if ( !abRet )
		{
			if( GetLastError () != ERROR_IO_PENDING)
			{
				char lastError[1024];
				FormatMessage(
						FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						lastError,
						1024,
						NULL);
				err_msg("%s", lastError);
				return 0;
			}
		}

		dwWait = WaitForMultipleObjects (1,&ov.hEvent,FALSE,int(tmout*1000));
		switch ( dwWait )
		{
			case WAIT_OBJECT_0:
			{
				DWORD dwMask;
				if (GetCommMask(_priv->hfd,&dwMask) )
				{
					if ( dwMask & EV_RXCHAR )
					{
						CloseHandle(ov.hEvent);
						return data_available(0);
					}
					else if ( dwMask & EV_TXEMPTY)
					{
						ResetEvent ( ov.hEvent );
						continue;
					}
				}
			}
			break;
		}//switch
	}
#endif
	int sz = data_available(0);
	return sz;
}

#endif
