#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>
#include <string>
#include <vector>


/*
    compile the .mm file as a normal .cc file with c++, to link you need to add -framework Cocoa
    NSSavePanel is to save files....
 
 
 have a look at
 http://stackoverflow.com/questions/10533517/with-nssavepanel-how-can-the-user-choose-a-specific-file-type-to-save
 
 
 */

std::vector<std::string> openFileDialog(const char * aTitle ,
                                        const char * aDefaultPathAndFile ,
                                        const std::vector<std::string> & filters) {
    
    int i=0;
    std::vector<std::string> fileList;
    // Create a File Open Dialog class.
    NSOpenPanel* openDlg = [NSOpenPanel openPanel];
    [openDlg setLevel:CGShieldingWindowLevel()];
    
    // Set array of file types
    NSMutableArray * fileTypesArray = [NSMutableArray array];
    for (i = 0;i < filters.size(); i++)
    {
        NSString * filt =[NSString stringWithUTF8String:filters[i].c_str()];
        [fileTypesArray addObject:filt];
    }
    
    // Enable options in the dialog.
    [openDlg setTitle: [NSString stringWithUTF8String:aTitle]];
    [openDlg setCanChooseFiles:YES];
    [openDlg setCanCreateDirectories:false];
    
    if (i)
        [openDlg setAllowedFileTypes:fileTypesArray];
    
    [openDlg setAllowsMultipleSelection:false];
    [openDlg setDirectoryURL:[NSURL URLWithString:[NSString stringWithUTF8String:aDefaultPathAndFile ] ] ];
    
    // Display the dialog box. If the OK pressed,
    // process the files.
    if ( [openDlg runModal] == NSOKButton ) {
        // Gets list of all files selected
        NSArray *files = [openDlg URLs];
        // Loop through the files and process them.
        for( i = 0; i < [files count]; i++ ) {
            // Do something with the filename.
            fileList.push_back(std::string([[[files objectAtIndex:i] path] UTF8String]));
        }
    }
    return fileList;
}

std::string saveFileDialog(const char * aTitle ,
                           const char * aDefaultPathAndFile ,
                           const std::vector<std::string> & filters)
{
    
    int i=0;
    std::string fileList;
    // Create a File Open Dialog class.
    NSSavePanel* saveDlg = [NSSavePanel savePanel];
    [saveDlg setLevel:CGShieldingWindowLevel()];
    
    // Set array of file types
    NSMutableArray * fileTypesArray = [NSMutableArray array];
    for (i = 0;i < filters.size(); i++)
    {
        NSString * filt =[NSString stringWithUTF8String:filters[i].c_str()];
        [fileTypesArray addObject:filt];
    }
    if (i)
        [saveDlg setAllowedFileTypes:fileTypesArray];
    
    // Enable options in the dialog.
    [saveDlg setTitle: [NSString stringWithUTF8String:aTitle]];
    [saveDlg setCanChooseFiles:false];
    [saveDlg setCanCreateDirectories:true];
    [saveDlg setDirectoryURL:[NSURL URLWithString:[NSString stringWithUTF8String:aDefaultPathAndFile ] ] ];
    
    // Display the dialog box. If the OK pressed,
    // process the files.
    if ( [saveDlg runModal] == NSOKButton ) {
        // Gets list of all files selected
        NSURL *ofile =   [saveDlg URL];
        NSString *ostring = [ofile path];
        fileList = [ostring UTF8String];
    }
    return fileList;
}