#include <iostream>
#include <string>
#include <ctime>
#include <cstdlib>
#include <csignal>
#include <getopt.h>
#include <alibavaClient.h>

bool do_run = true;
void INTcatcher(int sig)
{
    do_run = false;
}

int msleep(unsigned long milisec)
{
    struct timespec req={0};
    time_t sec=(int)(milisec/1000);
    milisec=milisec-(sec*1000);
    req.tv_sec=sec;
    req.tv_nsec=milisec*1000000L;
    return nanosleep(&req,&req);
}

int main(int argc, char **argv)
{
    Alibava client;
    int option_index;
    std::string uri("http://localhost:10000");
    int iAsync = 0;
    int mxEvt = 2500;
    int nRun = 200;
    int iMask = 3;
    static struct option long_options[] =
    {
     { "async", 0, &iAsync, 1},
     { "no-async", 0, &iAsync, 0},
     { "chip-mask", 1, 0, 'c'},
     { "nevts", 1, 0, 'n' },
     { "nrun", 1, 0, 'r'},
     { 0, 0, 0, 0 }
    };

    while ( 1)
    {
        char c;
        option_index = 0;
        c = getopt_long(argc, argv, "n:r:c:h", long_options, &option_index);
        if (c == -1)
            break;
        switch (c)
        {
            case 0:
                break;
            case 'n':
                mxEvt = atoi(optarg);
                break;
            case 'r':
                nRun = atoi(optarg);
                break;
            case 'c':
                iMask = atoi(optarg);
                if (iMask<0)
                    iMask = 1;
                if (iMask>3)
                    iMask =3;
                break;
            case 'h':
                std::cout << "soapClient: XXXX" << std::endl;
                return 0;

            case '?':
            default:
                std::cout << "Error " << argv[optind-1] << " ";
                if (optind<argc)
                    std::cout << argv[optind];
                std::cout << " [Ughhhh !!!!]" << std::endl;
                return 1;
        }
    }
    while (optind<argc)
        uri = argv[optind++];


    signal(SIGINT, INTcatcher);

    client.connect(uri.c_str());

    client.Reset();

    ns1__Status status;
    client.getStatus(status);
    std::cout << "Status:" << std::endl;
    std::cout << status << std::endl;

    if (status.state)
        client.stopRun();

    client.setParameter("chipMask", iMask);

    std::cout << "+++ Starting a Pedestal run" << std::endl;
    client.startPedestalRun(1000, false);
    client.getStatus(status);
    std::cout << "Status:" << std::endl;
    std::cout << status << std::endl;

    std::cout << "+++ Starting a Laser run" << std::endl;
    client.startLaserRun(1000, false);
    client.getStatus(status);
    std::cout << "Status:" << std::endl;
    std::cout << status << std::endl;


    client.setParameter("fileFormat", "hdf5");

    for (int irun=0; irun<nRun && do_run; irun++)
    {
        std::cout << "=== " << irun << " ===" << std::endl;
        if ( client.setDataFile("/tmp/data_file.h5") )
        {
            std::cout << "The server is still running... Sleeping for a while";
            msleep(100);
        }
        client.startSourceRun(mxEvt, iAsync);
        if (iAsync)
        {
            do {
                client.getStatus(status);
            } while ( !status.state );
        }
        std::cout << std::endl;
        for (bool do_run=true; do_run;)
        {
            msleep(1);
            if ( client.getStatus(status) )
                continue;

            std::cout << '\r' << status << std::flush;
            do_run = (status.state !=0);
        }
        if (iAsync)
        {
            while (true)
            {
                if ( client.getRunData() )
                {
                    std::cout << "No data found in server at rund end" << std::endl;
                    msleep(100);
                }
                else
                    break;
            }
        }
        client.getStatus(status);
        std::cout << "Status:" << std::endl;
        std::cout << status << std::endl;
        client.get_histogram(0)->Print(std::cout);
    }
    return 0;
}
