/*
 * test_data_socket.cc
 *
 *  Created on: Jan 19, 2014
 *      Author: lacasta
 */

#include <iostream>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <poll.h>
#include <unistd.h>
#include <cerrno>
#include <cstring>
#include <pthread.h>
#include <alibavaClient.h>
#include <ATimer.h>

#define SERVER_PORT 8000


std::string get_host_name(struct sockaddr_in *in_addr)
{
    char display[16] = {0};
    inet_ntop(AF_INET, &in_addr->sin_addr.s_addr, display, sizeof (display));
    return std::string(display);
}

int main(int argc, char **argv)
{
    const char *uri = "http://localhost:20000";
    Alibava alibava;
    pthread_t thread;
    int sockfd = 0, n = 0;
    unsigned short *recv=0;
    int nwords = 0;
    int header = 0;
    int data_size = 0;
    uint32_t iword;
    struct sockaddr_in serv_addr;
    struct addrinfo hints, *res;
    ns1__Status status;
    ATimer tim;

    if (argv[1])
        uri = argv[1];

    alibava.connect(uri);
    alibava.getStatus(status);

    std::cout << "Running a pedestal run" << std::endl;
    if (!status.state)
        alibava.startPedestalRun(1000,false);
    std::cout << "...done" << std::endl;

    if( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
        std::cout << "Could not open the socket" << std::endl;
        return 1;
    }

    memset(&serv_addr, 0, sizeof(serv_addr));
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;

    int iok = getaddrinfo("127.0.0.1", 0, &hints, &res);
    if (iok)
    {
        std::cout << "Call to getaddrinfo failed with rc=" << iok << std::endl;
        return -1;
    }
    if (res->ai_next)
    {
        std::cout << "Wildcard resolved to multiple addresses" << std::endl;
        freeaddrinfo(res);
        return -1;
    }
    memcpy(&serv_addr, res->ai_addr, res->ai_addrlen);
    serv_addr.sin_port = htons(SERVER_PORT);
    freeaddrinfo(res);

    std::cout << "Trying to connect to " << get_host_name(&serv_addr) << ":" << SERVER_PORT << std::endl;

    if ( connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0)
    {
        std::cout << "Could not connect to the server" << std::endl;
        close(sockfd);
        return 1;
    }


    // Start the DAQ. Call startRun in async mode
    tim.start();
    std::cout << "Start source run" << std::endl;
    alibava.startSourceRun(20000, true);


    // Waiting for data
    int rc, indata, timeout = 1000;
    int last_state=-1;
    while (true)
    {
        struct pollfd PS = { sockfd, POLLIN | POLLPRI | POLLERR| POLLHUP, 0 };
        iok = poll(&PS, 1, timeout);
        if (tim()>0.250)
        {
            tim.reset();
            alibava.getStatus(status);
            if (status.state)
            {
                if (last_state != status.state)
                    std::cout << "\nRun Started" << std::endl;
                std::cout << '\r' << status << "          " << std::flush;
            }
            else
            {
                if (last_state != status.state)
                {
                    std::cout << '\r' << status << "          " << std::flush;
                    std::cout << "\nRun Stopped\n" << std::endl;
                }
            }
            last_state = status.state;
        }
        if ( PS.revents & POLLHUP )
        {
            std::cout << "POLLHUP: Alibava stopped.?" << std::endl;
            break;
        }
        if ( PS.revents & POLLERR )
        {
            std::cout << "POLLERR" << std::endl;
            break;
        }
        if (iok==1)
        {
            /*
             * Here we read the data. We first get the header
             */
            do
            {
                rc = read(sockfd, (char *)&iword, sizeof(uint32_t));
                nwords = iword & 0xffff;
                header = iword >> 16;
                if (rc<=0)
                    return -1;
            }
            while (header != 0xcafe );

            /*
             * Prepare the data buffer
             */
            if (data_size < nwords)
            {
                data_size = nwords;
                if (recv)
                    delete [] recv;

                recv = new unsigned short[nwords];
            }

            /*
             * Read the data
             */
            size_t sz = nwords*sizeof(unsigned short);
            for (indata=0; indata<sz;)
            {
                rc = read(sockfd, (char *)recv, nwords*sizeof(unsigned short));
                if (rc<=0)
                {
                    std::cout << "Error while reading" << std::endl;
                    return -2;
                }
                indata += rc;
            }
        }
        else
        {
            if (iok==0)
            {
                if (status.state)
                    std::cout << "Timeout" << std::endl;
            }
            else if ( errno == EINTR )
            {
                std::cout << "Got an interrupt" << std::endl;
            }
            else
            {
                std::cout << "Something happened" << std::endl;
                break;
            }
        }
    }

    return 0;
}

