/*
 * timeStamp.cc
 *
 *  Created on: Jul 13, 2014
 *      Author: lacasta
 */

#include <iostream>
#include <cmath>
#include <TROOT.h>
#include <TCanvas.h>
#include <TApplication.h>
#include <TH1.h>
#include <TGraph.h>
#include <DataFileRoot.h>

class Tracer
{
    private:
        double nc, sx, sx2;
    public:
        Tracer();
        ~Tracer() {}

        void add(double x);
        double mean() const;
        double rms() const;
        double stdev() const;
        double npoints() const { return nc; }
};

Tracer::Tracer()
    : nc(0.0), sx(0.0), sx2(0.0)
{
}

void Tracer::add(double x)
{
    nc  ++;
    sx  += x;
    sx2 += x*x;
}

double Tracer::mean() const
{
    return nc>0.0 ? sx/nc : 0.0;
}

double Tracer::rms() const
{
    if (nc<2.)
        return 0.0;

    double m = mean();
    double rms = sx2/nc - m*m;

    return rms;
}

double Tracer::stdev() const
{
    return sqrt(rms());
}

int main(int argc, char **argv)
{
    const char *inp_file = argv[1];
    if (!inp_file)
    {
        std::cout << " I need an input file" << std::endl;
        return 1;
    }

    TApplication *app = new TApplication("theApp",&argc, argv);
    DataFileRoot *A = DataFileRoot::OpenFile(inp_file);
    int ievt;

    TGraph *gC = new TGraph();
    TGraph *gT = new TGraph();
    TGraph *gDead = new TGraph();
    double t0=-1, t, v;
    int ipoint = 0, jpoint=0;
    Tracer T;
    bool add_point;
    for (ievt=0; A->read_event()==0 ; ievt++)
    {
        t = A->clock_counter();
        v = (t-t0)*25.0e-9;
        add_point = true;
        if (t0>0)
        {
            if ( T.npoints() > 5 )
            {
                double stdev = T.stdev();
                double m = T.mean();
                double snr = fabs(v-m)/stdev;
                 if ( snr > 10.0 )
                    add_point = false;
            }
            if (add_point)
            {
                T.add(v);
                gC->SetPoint(ipoint, ievt, v);

                if (ipoint < 100)
                    gT->SetPoint(ipoint, ievt, A->time());
                ipoint++;
            }
            else
            {
                gDead->SetPoint(jpoint, jpoint, v);
                jpoint++;
            }
        }
        t0 = t;
    }

    std::cout << "Mean " << T.mean() << " stddev " << T.stdev() << " Freq " << 1.0/T.mean() << " Hz" << std::endl;

    TH1 *hst = new TH1D("hst", "Interevent time",
                     100,
                     T.mean() - 10.0*T.stdev(),
                     T.mean() + 10.0*T.stdev()
                     );
    for (int ih=0; ih<ipoint; ih++)
        hst->Fill( gC->GetY()[ih]);


    TCanvas *cnvs = new TCanvas();
    cnvs->Divide(1,4);

    cnvs->cd(1);
    gC->Draw("PA");

    cnvs->cd(2);
    gDead->Draw("LA");

    cnvs->cd(3);
    gT->Draw("LA");

    cnvs->cd(4);
    hst->Draw();

    app->Run(true);

    delete gC;
    delete gT;
    delete app;
    delete A;

    return 0;
}
